package de.ipk_gatersleben.ag_nw.centilib.gui;

import hep.aida.bin.DynamicBin1D;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.XYLineAndShapeRenderer;
import org.jfree.data.XYSeries;
import org.jfree.data.XYSeriesCollection;
//import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
//import org.jfree.data.xy.XYSeries;
//import org.jfree.data.xy.XYSeriesCollection;

import cern.colt.list.DoubleArrayList;
import cern.colt.list.IntArrayList;
import edu.uci.ics.jung.algorithms.scoring.VertexScorer;
import edu.uci.ics.jung.graph.Graph;

/**
 * This class was implemented for Vanted again, since Vanted comes with an older version of
 * JFreeChart.
 * Dialog with the centrality distribution.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 */
public class CentralityDistributionDialog<V,E> extends JDialog {

    public CentralityDistributionDialog(Frame owner) throws HeadlessException {
    	/*
    	 * a new Frame is generated to allow this dialog to get in the background
    	 * if another window is selected (since java 1.6 the dialogs stay in front of
    	 * the parent frame)
    	 */
        super(new JFrame(),"",false);//owner, "", false);
        
        getContentPane().setLayout(new BorderLayout());
        setSize(new Dimension(800, 600));
        setTitle("Centrality Distribution Plot");
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    }

    private JButton createCloseButton() {

        final CentralityDistributionDialog<V,E> cdd = this;
        JButton okButton = new JButton(new AbstractAction("Close") {
            public void actionPerformed(ActionEvent arg0) {
                cdd.setVisible(false);
            }
        });

        return okButton;
    }

    ChartPanel createPlot(String centName, VertexScorer<V,Double> cents, Graph<V,E> graph) {

        XYSeries xys = transformCentralitiesIntoDataset(cents,graph);

        JFreeChart chart = ChartFactory.createXYLineChart(centName + " Centrality Distribution", "Centrality Value",
                "Number of Vertices", new XYSeriesCollection(xys), PlotOrientation.VERTICAL, false, false, false);

        configurePlot(chart);

        ChartPanel chartPanel = new ChartPanel(chart);

        return chartPanel;
    }

    private void configurePlot(JFreeChart chart) {
        XYPlot plot = (XYPlot) chart.getPlot();

        XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
        renderer.setSeriesLinesVisible(0, false);
        renderer.setSeriesShapesVisible(0, true);
        plot.setRenderer(renderer);
    }

    private XYSeries transformCentralitiesIntoDataset(VertexScorer<V, Double> scorer, Graph<V,E> graph) {
        DynamicBin1D bin = new DynamicBin1D();
        for (V v : graph.getVertices()) {
            Double score = scorer.getVertexScore(v);
            if(score != null)
            	bin.add(score);
        }

        DoubleArrayList dal = new DoubleArrayList();
        IntArrayList ial = new IntArrayList();
        bin.frequencies(dal, ial);

        XYSeries xys = new XYSeries("1");

        for (int i = 0; i < dal.size(); i++) {
            xys.add(dal.get(i), ial.get(i));
        }
        return xys;
    }

    public void activate(String centName, VertexScorer<V,Double> cents, Graph<V,E> graph) {

        getContentPane().removeAll();
        getContentPane().add(createPlot(centName, cents,graph), BorderLayout.CENTER);
        getContentPane().add(createCloseButton(), BorderLayout.SOUTH);
        validate();
        setVisible(true);
    }
}
