package de.ipk_gatersleben.ag_nw.centilib.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.statistics.HistogramDataset;

import edu.uci.ics.jung.algorithms.scoring.VertexScorer;
import edu.uci.ics.jung.graph.Graph;

/**
 * This class was implemented for Vanted again, since Vanted comes with an older version of
 * JFreeChart.
 * Dialog with the centrality histogram.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 */

public class CentralityHistogramDialog<V,E> extends JDialog {

	public CentralityHistogramDialog(Frame owner) throws HeadlessException {
		/*
    	 * a new Frame is generated to allow this dialog to get in the background
    	 * if another window is selected (the new dialogs will stay in front of
    	 * the parent frame)
    	 */
        super(new JFrame(),"",false);//owner, "", false);

        getContentPane().setLayout(new BorderLayout());
        setSize(new Dimension(800, 600));
        setTitle("Centrality Histogram Plot");
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    }

    private JButton createCloseButton() {

        final CentralityHistogramDialog<V,E> chd = this;
        JButton okButton = new JButton(new AbstractAction("Close") {
            public void actionPerformed(ActionEvent arg0) {
                chd.setVisible(false);
            }
        });

        return okButton;
    }

    private ChartPanel createPlot(String centName, VertexScorer<V,Double> cents,Graph<V,E> graph) {

        HistogramDataset hd = transformCentralitiesIntoDataset(cents,graph);

        JFreeChart chart = ChartFactory.createHistogram(centName + " Centrality Histogram", "Centrality Value",
                "Number of Vertices", hd, PlotOrientation.VERTICAL, false, false, false);

        configurePlot(chart);

        ChartPanel chartPanel = new ChartPanel(chart);

        return chartPanel;
    }

    private void configurePlot(JFreeChart chart) {
        // XYPlot plot = (XYPlot) chart.getPlot();
    }

    private HistogramDataset transformCentralitiesIntoDataset(VertexScorer<V,Double> cents, Graph<V,E> graph) {

        HistogramDataset hd = new HistogramDataset();

        double[] values = new double[graph.getVertexCount()];
        int i = 0;
        double max = Double.NEGATIVE_INFINITY,
        	min = Double.POSITIVE_INFINITY;
        for (V v : graph.getVertices()){
        	if(cents.getVertexScore(v)!= null){
        		values[i] = cents.getVertexScore(v);
        		
        		//TODO recognize negative values (Closeness Vitality)
        		if(values[i] > max && values[i] != Double.POSITIVE_INFINITY)
        			max = values[i];
        		if(values[i] < min)
        			min = values[i];
        		i++;
        	}
        }

        if(max<1.0)
        	max = 10;
        else{
        	max -= min;
        	max++;
        }

        hd.addSeries("2", values, (int)max);
        return hd;
    }
    
    public void activate(String centName, VertexScorer<V,Double> cents, Graph<V,E> graph) {

        getContentPane().removeAll();
        getContentPane().add(createPlot(centName, cents, graph), BorderLayout.CENTER);
        getContentPane().add(createCloseButton(), BorderLayout.SOUTH);
        validate();

        setVisible(true);
    }
}
