package plugin;

import giny.model.Edge;
import giny.model.GraphObject;
import giny.model.GraphPerspectiveChangeEvent;
import giny.model.GraphPerspectiveChangeListener;
import giny.model.Node;
import giny.view.EdgeView;
import giny.view.NodeView;

import java.awt.geom.Point2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.DecimalFormat;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.SwingConstants;

import org.apache.commons.collections15.Transformer;

import cytoscape.CyNetwork;
import cytoscape.CyNetworkEvent;
import cytoscape.CyNetworkListener;
import cytoscape.Cytoscape;
import cytoscape.data.CyAttributes;
import cytoscape.data.SelectEvent;
import cytoscape.data.SelectEventListener;
import cytoscape.view.CyNetworkView;
import cytoscape.view.cytopanels.CytoPanelImp;
import cytoscape.visual.VisualStyle;
import de.ipk_gatersleben.ag_nw.centilib.plugin.CentiLibInteractions;
import de.ipk_gatersleben.ag_nw.centilib.plugin.CentiLibPlugin;
import de.ipk_gatersleben.ag_nw.centilib.plugin.DoubleAttribute;
import ding.view.DGraphView;
import edu.uci.ics.jung.graph.Graph;


public class MyAllroundListener implements CyNetworkListener,
		GraphPerspectiveChangeListener,
		PropertyChangeListener,
		SelectEventListener,
		CentiLibInteractions<CyNetwork,Node, Edge>{

	
	public static final String PANEL_NAME = "CentiLib";
	
	public static final int PANEL_DIRECTION = SwingConstants.WEST;


	private CentiLibPlugin<CyNetwork,Node,Edge> centilib;
    

	
	
	public MyAllroundListener(){
		
		Transformer<CyNetwork,String> graphT = new Transformer<CyNetwork,String>(){
			public String transform (CyNetwork graph){
				return graph.getTitle();//getIdentifier();
			}
		};
		
		Transformer<Node,String> nodeT = new Transformer<Node,String>(){
					public String transform(Node node){
						return node.getIdentifier();
					}
				};
				
		centilib = new CentiLibPlugin<CyNetwork,Node,Edge>(
				//Cytoscape.getDesktop(),
				this,
				graphT,
				nodeT);
		
		
		CytoPanelImp ctrlPanel = (CytoPanelImp) Cytoscape.getDesktop().getCytoPanel(PANEL_DIRECTION);
        ctrlPanel.add(PANEL_NAME,centilib.getPanel());

        Cytoscape.getDesktop().getSwingPropertyChangeSupport().addPropertyChangeListener(this);
		
	}
	
	@Override
	public void onCyNetworkEvent(CyNetworkEvent event) {
		centilib.graphFocused(Cytoscape.getCurrentNetwork());
	}

	@Override
	public void graphPerspectiveChanged(GraphPerspectiveChangeEvent event) {
		
		if(
			event.getType() == GraphPerspectiveChangeEvent.EDGES_HIDDEN_TYPE ||
			event.getType() == GraphPerspectiveChangeEvent.EDGES_RESTORED_TYPE ||
			event.getType() == GraphPerspectiveChangeEvent.NODES_HIDDEN_TYPE ||
			event.getType() == GraphPerspectiveChangeEvent.NODES_RESTORED_TYPE
		){
			centilib.preGraphChanged(getCurrentGraph());
			//centilib.postGraphChanged(getCurrentGraph());
			System.out.println("removed");
		}
	}
	
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		
		Cytoscape.getCurrentNetwork().addCyNetworkListener(this);
		
		Cytoscape.getCurrentNetwork().removeGraphPerspectiveChangeListener(this);
		Cytoscape.getCurrentNetwork().addGraphPerspectiveChangeListener(this);
		
		Cytoscape.getCurrentNetwork().removeSelectEventListener(this);
		Cytoscape.getCurrentNetwork().addSelectEventListener(this);
		
		centilib.graphFocused(Cytoscape.getCurrentNetwork());
	}
	
	@Override
	public void onSelectEvent(SelectEvent arg0) {
		
		centilib.nodesSelected((Collection<Node>)Cytoscape.getCurrentNetwork().getSelectedNodes());
	}

	
/*	private Map<String, LinkedList<Double> > getCentralityAttributes(CyNetwork graph){
		
		Collection<String> allCents = CentralityHandler.getCentList();
		CyAttributes attributes = Cytoscape.getNodeAttributes();		
		Map<String,LinkedList<Double> > availableCents = new HashMap<String,LinkedList<Double> >();		
		
		for(String attr : attributes.getAttributeNames())
			for(String cent : allCents){
				if(attr.contains(cent)){
					availableCents.put(attr, new LinkedList<Double>());
					List<Node> temp = graph.nodesList();
					
					if(isModified.get(graph) == null || !isModified.get(graph)){
						for(Node n : temp)
							try{
								if(attributes.getDoubleAttribute(n.getIdentifier(), attr) == null){
							
									availableCents.remove(attr);
									break;
								}
								else
									availableCents.get(attr).add(attributes.getDoubleAttribute(n.getIdentifier(), attr));
							}
							catch(Exception e){
								availableCents.remove(attr);
								break;
							}
					}
						
				}//fi
			}//rof
		return availableCents;
	}

*/
	@Override
	public boolean addCentralityAttribute(Node node, String centrality,
			Double value) {
		if(node == null || centrality == null)
			return false;
		
		CyAttributes attributes = Cytoscape.getNodeAttributes();
		DecimalFormat df = new DecimalFormat("#.#####");
		try{
			attributes.setAttribute(node.getIdentifier(), centrality, Double.valueOf(df.format(value).replace(',', '.')));
		}
		catch(IllegalArgumentException e){
			return false;
		}
		
		return true;
	}

	@Override
	public CyNetwork createSubGraph(CyNetwork graph, Set<Node> nodes, Set<Edge> edges,
			String name) {
		
		if(nodes == null || edges == null || name == null)
			return null;
		
		CyNetworkView origView = Cytoscape.getCurrentNetworkView();
		VisualStyle vs = Cytoscape.getVisualMappingManager().getVisualStyle(); 

		CyNetwork new_network = Cytoscape.createNetwork(nodes, edges, name);
		
		//removeCentralityAttributes(new_network);
		
		// only clone the view if one actually exists
		if ( origView == null || origView == Cytoscape.getNullNetworkView() )
			return null;

		CyNetworkView newView = Cytoscape.getNetworkView(new_network.getIdentifier());
		if ( newView != null || newView != Cytoscape.getNullNetworkView() ) {

        	// Use nodes as keys because they are less volatile than views...
	        Iterator ni = origView.getGraphPerspective().nodesIterator();
			while (ni.hasNext()) {
				Node n = (Node) ni.next();

				NodeView onv = origView.getNodeView(n);
				NodeView nnv = newView.getNodeView(n);

				nnv.setXPosition(onv.getXPosition());
				nnv.setYPosition(onv.getYPosition());
			}

			newView.setZoom(origView.getZoom());
			Point2D origCenter = ((DGraphView)origView).getCenter();
			((DGraphView)newView).setCenter(origCenter.getX(), origCenter.getY());

			// set edge anchors and bends
	        Iterator ei = origView.getGraphPerspective().edgesIterator();
			while (ei.hasNext()) {
				Edge ee = (Edge) ei.next();

				EdgeView oev = origView.getEdgeView(ee);
				EdgeView nev = newView.getEdgeView(ee);

				nev.getBend().setHandles(oev.getBend().getHandles());
				nev.getBend().setHandles(oev.getBend().getHandles());

				nev.setLineType( oev.getLineType() );
			}

		}
		
		Cytoscape.getVisualMappingManager().setVisualStyle(vs);
		return new_network;
	}

	@Override
	public void deleteCentralityAttribute(Node node, String centrality) {

		try{
			Cytoscape.getNodeAttributes().deleteAttribute(node.getIdentifier(), centrality);		
		}
		catch(Exception e){}
	}

	@Override
	public CyNetwork getCurrentGraph() {
		return Cytoscape.getCurrentNetwork();
	}

	@Override
	public Set<CyNetwork> getGraphs() {
		return Cytoscape.getNetworkSet();
	}

	@Override
	public Set<DoubleAttribute<Edge>> getNumericEdgeAttributes(CyNetwork graph) {

		if(graph == null || graph.getEdgeCount() == 0)
			return null;
		
		CyAttributes allAttr = Cytoscape.getEdgeAttributes();
		Set<DoubleAttribute<Edge> > possibleAttr = new HashSet<DoubleAttribute<Edge> >();
		boolean isPossible = true;

		for(String attr : allAttr.getAttributeNames()){
			isPossible = true;
			if(allAttr.getType(attr) != CyAttributes.TYPE_FLOATING && 
					allAttr.getType(attr) != CyAttributes.TYPE_INTEGER)
				continue;
				
			for(Edge e : (List<Edge>) graph.edgesList()){
				try{
					if(allAttr.getType(attr) == CyAttributes.TYPE_INTEGER)
						if(allAttr.getIntegerAttribute(e.getIdentifier(), attr) == null){
							isPossible = false;
							break;
						}
					
					if(allAttr.getType(attr) == CyAttributes.TYPE_FLOATING)
						if(allAttr.getDoubleAttribute(e.getIdentifier(), attr) == null){
							isPossible = false;
							break;
						}
				}catch(ClassCastException exc){
					isPossible = false;
					break;
				}
			}
				
			if(isPossible)
				possibleAttr.add(new CytoAttribute<Edge>(attr, allAttr));
		}
		if(possibleAttr.size() == 0)
			return null;
		else return possibleAttr;
	}

	@Override
	public Set<DoubleAttribute<Node>> getNumericNodeAttributes(CyNetwork graph) {
		if(graph == null || graph.getEdgeCount() == 0)
			return null;
		
		CyAttributes allAttr = Cytoscape.getNodeAttributes();
		Set<DoubleAttribute<Node> > possibleAttr = new HashSet<DoubleAttribute<Node> >();
		boolean isPossible = true;
		
		for(String attr : allAttr.getAttributeNames()){
			isPossible = true;
			if(allAttr.getType(attr) != CyAttributes.TYPE_FLOATING && 
					allAttr.getType(attr) != CyAttributes.TYPE_INTEGER)
				continue;
		
			for(Node n : (List<Node>) graph.nodesList()){
/*				try{
					if(allAttr.getIntegerAttribute(n.getIdentifier(), attr) != null){
						continue;
					}
				}catch(Exception exc){}
	*/			
				try{
					if(allAttr.getDoubleAttribute(n.getIdentifier(), attr) == null){
						isPossible = false;
						break;
					}
				}
				catch(ClassCastException e){
					isPossible = false;
					break;
				}
			}
				
			if(isPossible)
				possibleAttr.add(new CytoAttribute<Node>(attr, allAttr));
		}
		if(possibleAttr.size() == 0)
			return null;
		else return possibleAttr;
	}

	@Override
	public Graph<Node, Edge> getWrapper(CyNetwork graph) {
		return new CyNetworkWrapper<Node, Edge>(graph);
	}

	@Override
	public void selectGraph(CyNetwork graph) {
		if(graph == null)
			return;
		
		Cytoscape.getDesktop().setFocus(graph.getIdentifier());	
	}

	@Override
	public void selectNodesOfCurrentGraph(Set<Node> vertices) {
		
		CyNetwork network= Cytoscape.getCurrentNetwork();
		if(network == null){
			return;
		}
		
		network.unselectAllNodes();
		network.unselectAllEdges();
		network.setSelectedNodeState(vertices,true);
		Cytoscape.getCurrentNetworkView().redrawGraph(true, false);
	}
	
	private class CytoAttribute<T extends GraphObject> implements DoubleAttribute<T>{
		
		private String name = "";
		private CyAttributes attributes;
		
		public CytoAttribute(String name, CyAttributes attributes){
			this.name = name;
			this.attributes = attributes;
		}

		@Override
		public Double getValue(T element) {
			
			if(element == null)
				return null;
			
			Object attr = attributes.getAttribute(element.getIdentifier(), name);
			if(attr != null && attr instanceof Number)
				return ((Number)attr).doubleValue();
			
			return null;
		}
		
		@Override
		public String toString(){
			return name;
		}

		@Override
		public String getName() {
			return name;
		}
	}
}
