package de.ipk_gatersleben.ag_nw.centilib;


import java.awt.Cursor;
import java.util.Collection;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.apache.commons.collections15.Transformer;

import de.ipk_gatersleben.ag_nw.centilib.gui.CentralityDistributionDialog;
import de.ipk_gatersleben.ag_nw.centilib.gui.CentralityHistogramDialog;
import de.ipk_gatersleben.ag_nw.centilib.centralities.VertexCentrality;
import de.ipk_gatersleben.ag_nw.centilib.gui.CentiLibPanel;
import de.ipk_gatersleben.ag_nw.centilib.gui.GraphFileChooser;
import de.ipk_gatersleben.ag_nw.centilib.gui.utils.JGetNumber;
import de.ipk_gatersleben.ag_nw.centilib.plugin.ApplicationInterface;
import de.ipk_gatersleben.ag_nw.centilib.utils.CentiLibFileWrapper;
import de.ipk_gatersleben.ag_nw.centilib.utils.ErrorHandler;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;
import edu.uci.ics.jung.algorithms.shortestpath.Distance;
import edu.uci.ics.jung.graph.Graph;

/**
 * The application class for CentiLib.
 * 
 * @author Johannes Graessler
 * @author Dirk Koschuetzki
 */
public class CentiLib<V,E> {

    // Our constant for comparing doubles.
    public static final double EPSILON = 1e-7;

    public static final String WAIT_STRING = "Please wait...";

    public static final String CONFIRMATION_WAIT = "Waiting for confirmation...";

    public static final String DIALOG_OPEN = "A dialog is open.";

    
    private String LASTLONGOPERATIONTEXT = "";
    
    private String LASTLONGPROGRESSTEXT = "";
      
    private JFrame parentFrame;
    
    private CentiLibPanel<V,E> Panel;
    
    private ApplicationInterface<V,E> application;

    private Graph<V,E> currentGraph;

    /*
     * Transformer for getting the names of the nodes
     */
    private Transformer<V,String> vertexTransformer;
    
    private String currentGraphName;

    private VertexCentrality<V,E> currentRanker;

    private String currentRanking;
    
   

    
    public CentiLib(ApplicationInterface<V,E> application, Transformer<V,String> vertexTransformer){
    	this.parentFrame = new JFrame();//parent;
    	this.vertexTransformer = vertexTransformer;
    	this.application = application;
        ErrorHandler.init(parentFrame);
        GraphFileChooser.init(parentFrame);
        Panel = new CentiLibPanel<V,E>(this);
        //CentralityHandler.getCentList();
    }

         
    public CentiLibPanel<V,E> getPanel(){
    	return Panel;
    }
    
    public Transformer<V,String> getVertexLabelTransformer(){
    	return vertexTransformer;
    }
    
    public void refreshPanel(){
    	currentRanker = GraphCachingUtils.getLastScorer(currentGraph);
    	currentRanking = GraphCachingUtils.getLastScorerName(currentGraph);
    	Panel.setGraph(currentGraph);
    	Panel.setRanker(currentRanker, currentRanking);
    }
    
    public void setHasChanged(){
    	if(currentGraph != null){
    		GraphCachingUtils.remove(currentGraph);
    		setCurrentGraph(currentGraph,currentGraphName);
    	}
    }
    
    public Distance<V> getDistance(){
    	return GraphCachingUtils.getDistance(currentGraph);
    }

    public void setCurrentGraph(Graph<V,E> graph, String graphName) {
    	
    	if(graph != null && graph == currentGraph && graphName.equals(currentGraphName))
    		return;
    	
    	this.currentGraph = graph;
        this.currentGraphName = graphName;
        GraphCachingUtils.setGraphName(graph,graphName);
    	currentRanker = GraphCachingUtils.getLastScorer(graph);
    	currentRanking = GraphCachingUtils.getLastScorerName(graph);
    	Panel.setGraph(currentGraph);
    	//refreshPanel();
    }
    
    public Collection<V> getSelectedVertices() {
        return Panel.getSelectedVertices();
    }

    public Graph<V,E> getCurrentGraph() {
        return currentGraph;
    }

    public void deleteGraph(Graph<V,E> graph){
    	if(graph != null && graph == currentGraph)
    		setCurrentGraph(null,"");
    	
    	GraphCachingUtils.remove(graph);
    }
    
    public VertexCentrality<V,E> getCurrentRanker() {
        return currentRanker;
    }

    public void setCurrentRanker(VertexCentrality<V,E> ranker, String rankingName) {
        this.currentRanker = ranker;
        this.currentRanking = rankingName;
        GraphCachingUtils.addScorer(currentGraph, ranker, rankingName);
        Panel.setRanker(currentRanker,rankingName);
        //refreshPanel();
    }

    public String getCurrentRanking() {
        return currentRanking;
    }

    public String getCurrentGraphName() {
        return currentGraphName;
    }

    public ApplicationInterface<V,E> getAppInterface(){
    	return application;
    }
    
    public void startLongRunningOperation(String operation, String progress) {
        parentFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        //parentFrame.setEnabled(false);

        LASTLONGOPERATIONTEXT = operation;
        LASTLONGPROGRESSTEXT = progress;
        Panel.setOperationString(operation);
        Panel.setProgressString(progress);
        Panel.startProgressBar();
    }

    public void holdLongRunningOperation(String operation, String progress) {
        parentFrame.setEnabled(true);
        parentFrame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

        Panel.setOperationString(operation);
        Panel.setProgressString(progress);
        Panel.stopProgressBar();
    }

    public void continueLongRunningOperation() {
        // Does not seem to work. Either disabling the frame or switching the
        // cursor. :-(
        if(LASTLONGPROGRESSTEXT.equals("")){
        	finishedLongRunningOperation();
        	return;
        }
        	
        parentFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        //parentFrame.setEnabled(false);
        
        Panel.setOperationString(LASTLONGOPERATIONTEXT);
        Panel.setProgressString(LASTLONGPROGRESSTEXT);
        Panel.startProgressBar();

    }

    public void finishedLongRunningOperation() {
        parentFrame.setEnabled(true);
        parentFrame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        
        LASTLONGOPERATIONTEXT = "";
        LASTLONGPROGRESSTEXT = "";
        Panel.stopProgressBar();
        Panel.setOperationString("");
        Panel.setProgressString("");
    }

    public void showInformationDialog(String dialogTitle, String dialogMessage) {
        holdLongRunningOperation(DIALOG_OPEN, CONFIRMATION_WAIT);
        JOptionPane.showMessageDialog(parentFrame, dialogMessage, dialogTitle, JOptionPane.INFORMATION_MESSAGE);
        continueLongRunningOperation();
    }

    public void showCentralityHistogramDialog() {
        CentralityHistogramDialog<V,E> chd = new CentralityHistogramDialog<V,E>(parentFrame);
        chd.activate(currentRanking, currentRanker,currentGraph);
    }

    public void showCentralityDistributionDialog() {
        CentralityDistributionDialog<V,E> cdd = new CentralityDistributionDialog<V,E>(parentFrame);
        cdd.activate(currentRanking, currentRanker,currentGraph);
    }

    public String askForString(String question, String defaultValue) {
        holdLongRunningOperation(DIALOG_OPEN, CONFIRMATION_WAIT);
        String result = JOptionPane.showInputDialog(parentFrame, question, defaultValue);
        continueLongRunningOperation();
        return result;
    }

    public Integer askForInteger(String question, String defaultValue, int min, int max) {
        holdLongRunningOperation(DIALOG_OPEN, CONFIRMATION_WAIT);
        Integer result = JGetNumber.getInteger(parentFrame, question, defaultValue, min, max);
        continueLongRunningOperation();
        return result;
    }

    public Integer askForInteger(String question, String defaultValue) {
        holdLongRunningOperation(DIALOG_OPEN, CONFIRMATION_WAIT);
        Integer result = JGetNumber.getInteger(parentFrame, question, defaultValue);
        continueLongRunningOperation();

        return result;

    }

    public boolean askForBoolean(String question) {
        holdLongRunningOperation(DIALOG_OPEN, CONFIRMATION_WAIT);
        boolean result = JGetNumber.getBoolean(parentFrame, question);
        continueLongRunningOperation();

        return result;
    }

    public Double askForDouble(String question, String defaultValue, double min, double max) {
        holdLongRunningOperation(DIALOG_OPEN, CONFIRMATION_WAIT);
        Double result = JGetNumber.getDouble(parentFrame, question, defaultValue, min, max);
        continueLongRunningOperation();

        return result;
    }

    public Double askForDouble(String question, String defaultValue) {
        holdLongRunningOperation(DIALOG_OPEN, CONFIRMATION_WAIT);
        Double result = JGetNumber.getDouble(parentFrame, question, defaultValue);
        continueLongRunningOperation();
        return result;
    }

    public CentiLibFileWrapper showOpenVectorDialog() {
        holdLongRunningOperation(DIALOG_OPEN, CONFIRMATION_WAIT);
        CentiLibFileWrapper selectionResult = GraphFileChooser.showOpenVectorDialog();
        continueLongRunningOperation();
        return selectionResult;
    }

    public CentiLibFileWrapper showSaveVectorDialog(String recommendedName) {
        holdLongRunningOperation(DIALOG_OPEN, CONFIRMATION_WAIT);
        CentiLibFileWrapper selectionResult = GraphFileChooser.showSaveVectorDialog(recommendedName);
        
		while(selectionResult!= null && selectionResult.getSelectedFile().exists()){
			if(askForBoolean("The selected File already exists!\n" +
					"Do you want to overwrite it?"))
					break;
			recommendedName = selectionResult.getSelectedFile().getName();
			selectionResult = GraphFileChooser.showSaveTableDialog(recommendedName);
		}    	
        continueLongRunningOperation();
        return selectionResult;
    }
    
    public CentiLibFileWrapper showSaveSelectionDialog(String recommendedName) {
        holdLongRunningOperation(DIALOG_OPEN, CONFIRMATION_WAIT);
        CentiLibFileWrapper selectionResult = GraphFileChooser.showSaveSelectionDialog(recommendedName);
        continueLongRunningOperation();
        return selectionResult;
    }
    
    public CentiLibFileWrapper showSaveTableDialog(String recommendedName){
    	holdLongRunningOperation(DIALOG_OPEN, CONFIRMATION_WAIT);
    	CentiLibFileWrapper selectionResult = GraphFileChooser.showSaveTableDialog(recommendedName);
    	
		while(selectionResult!= null && selectionResult.getSelectedFile().exists()){
			if(askForBoolean("The selected File already exists!\n" +
					"Do you want to overwrite it?"))
					break;
			recommendedName = selectionResult.getSelectedFile().getName();
			selectionResult = GraphFileChooser.showSaveTableDialog(recommendedName);
		}    	
    	continueLongRunningOperation();
    	return selectionResult;
    }

    public void setSelectedVertices(Collection<V> selected){
    	Panel.setSelectedVertices(selected);
    }
}