package de.ipk_gatersleben.ag_nw.centilib.centralities;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.collections15.BidiMap;

import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesSPData;
import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesSPListener;
import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesVertexData;
import edu.uci.ics.jung.algorithms.util.Indexer;
import edu.uci.ics.jung.graph.Graph;


/**
 * 
 * This implementation is wrong
 * @author Johannes Graessler
 *
 * @param <V>
 * @param <E>
 */
public class BrandesCentroidValues<V, E> extends VertexCentrality<V,E> 
		implements BrandesSPListener<V, E> {

	protected int[][] gammaValues;
	
	protected int numVertices;
	
	protected BidiMap<V,Integer> indexer;
	
	protected Map<V,Map<V,BrandesVertexData<E> > > visited = new HashMap<V, Map<V,BrandesVertexData<E> > >();
	
	public BrandesCentroidValues(Graph<V,E> graph) {
		super("CentroidValues",graph);
		this.numVertices = graph.getVertexCount();
		this.indexer = Indexer.create(graph.getVertices());
		gammaValues = createEmptyMatrix();
	}

	@Override
	public void finished() {
		
        int[][] fValues = computeFValue(gammaValues);

        for (V w : graph.getVertices()){
            int i = indexer.get(w);
            output.put(w, extractMinFValue(fValues, i));
        }

	}
	
	@Override
	public void computeDependencies(BrandesSPData<V, E> stepData) {

		if(this.graph != stepData.getGraph() || output.size() != 0)
			return;
		
		if(graph.getVertexCount() != stepData.getStack().size())
			throw new IllegalArgumentException("Graph is not strong connected");
		
		V source = stepData.getStack().getLast();
		Map<V,Integer> dependency = new HashMap<V,Integer>();
		
		for(V v : graph.getVertices())
			dependency.put(v, 0);
		
		//instead of pop() we iterate through the LinkedList
		for(V target : stepData.getStack()){
			if(target == source)
				break;
			
			for(V v: visited.keySet()){
				if(v == target)
					continue;
				double vTargetDist = visited.get(v).get(target).distance;
				double vSourceDist = visited.get(v).get(source).distance;
				double sTargetDist = stepData.getVertexData().get(target).distance;
				
				if(vTargetDist < vSourceDist)
					if(sTargetDist< vTargetDist)
						gammaValues[indexer.get(source)][indexer.get(v)]++;
					if(sTargetDist > vTargetDist)
						gammaValues[indexer.get(v)][indexer.get(source)]++;
			}
			
			for(E edge : stepData.getVertexData().get(target).incomingEdges){
				V opp = graph.getOpposite(target, edge);
				
				if(opp == target)
					continue;
				int nrSuccessors = dependency.get(opp);
				nrSuccessors += dependency.get(target)+1;
				dependency.put(opp, nrSuccessors);
			}
			if(target != source)
				gammaValues[indexer.get(target)][indexer.get(source)] += dependency.get(target);
		}
		
		visited.put(source, stepData.getVertexData());
		
	}

	@Override
	public Double getVertexScore(V v) {
		
		if(v == null)
			return null;
		
		return output.get(v);
	}

	private int[][] createEmptyMatrix() {
        int[][] gammaValues = new int[numVertices][numVertices];
        for (int i = 0; i < numVertices; i++) {
            for (int j = 0; j < numVertices; j++) {
                gammaValues[i][j] = 0;
            }
        }
        return gammaValues;
    }	
    
   private int[][] computeFValue(int[][] gammaValues) {
        int[][] f = new int[numVertices][numVertices];
        for (int u = 0; u < numVertices; u++) {
            for (int v = 0; v < numVertices; v++) {
                f[u][v] = gammaValues[u][v] - gammaValues[v][u];
            }
        }
        return f;
    }

    private Double extractMinFValue(int[][] f, int u) {
        Double result = Double.MAX_VALUE;
        for (int i = 0; i < numVertices; i++) {
            if (f[u][i] < result && u != i) {
                result = Double.valueOf(Integer.toString(f[u][i]));
            }
        }
        return result;
    }
 
}
