package de.ipk_gatersleben.ag_nw.centilib.centralities;

import java.util.Map;

import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesSPData;
import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesSPListener;
import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesVertexData;
import edu.uci.ics.jung.graph.Graph;

public class BrandesCloseness<V, E> extends VertexCentrality<V,E> 
		implements BrandesSPListener<V, E> {

	
	private boolean finished = false;
	
	
	
	public BrandesCloseness(Graph<V,E> graph) {
		super("Closeness",graph);
	}

	@Override
	public void finished() {finished = true;}

	@Override
	public void computeDependencies(BrandesSPData<V, E> stepData) {
		
		if(this.graph != stepData.getGraph() || finished)
			return;
		
		Map<V,BrandesVertexData<E> > vertexData = stepData.getVertexData();
		double result = 0.0;
		
		if(stepData.getStack().size() != graph.getVertexCount())
			result = Double.MAX_VALUE;
		else
			for(V target : stepData.getStack())
				result += vertexData.get(target).distance;
			
		assert(result != 0);
		output.put(stepData.getStack().getLast(), 1/result);
	}

	@Override
	public Double getVertexScore(V v) {
		if(v == null)
			return null;
		return output.get(v);
	}

}
