package de.ipk_gatersleben.ag_nw.centilib.centralities;

import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesSPData;
import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesSPListener;
import edu.uci.ics.jung.graph.Graph;

public class BrandesEccentricity<V,E> extends VertexCentrality<V,E> 
		implements BrandesSPListener<V, E> {

	protected boolean finished = false;
	
	
	public BrandesEccentricity(Graph<V,E> graph) {
		super("Eccentricity",graph);
	}

	@Override
	public void finished() {finished = true;}

	@Override
	public void computeDependencies(BrandesSPData<V, E> stepData) {
		if(graph != stepData.getGraph() || finished)
			return;
		
		V source = stepData.getStack().getLast();
		V widest = stepData.getStack().getFirst();
		Double newDist;
		if(widest == source)
			newDist = Double.POSITIVE_INFINITY;
		
		else
			newDist = stepData.getVertexData().get(widest).distance;
		
		//if graph is not connected (that is strong connected)
		if(stepData.getStack().size() != graph.getVertexCount())
			newDist = Double.POSITIVE_INFINITY;
		
		assert(newDist != 0);

		output.put(source, 1/newDist);
		
	}

	@Override
	public Double getVertexScore(V v) {
		if(v == null)
			return null;
		return output.get(v);
	}

}
