package de.ipk_gatersleben.ag_nw.centilib.centralities;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesSPData;
import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesSPListener;
import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesVertexData;
import edu.uci.ics.jung.graph.Graph;

public class BrandesStress<V,E> extends VertexCentrality<V,E> 
		implements BrandesSPListener<V, E>	{

	
	protected Map<V,Double> currentOut= new HashMap<V,Double>();
	
	
	public BrandesStress(Graph<V,E> graph) {
		super("Stress",graph);
		for(V v : graph.getVertices())
			currentOut.put(v, 0.0);
	}

	@Override
	public void finished() {
		output = currentOut;
 	}

	@Override
	public void computeDependencies(BrandesSPData<V, E> stepData) {
		if(output.size() != 0 || this.graph != stepData.getGraph())
			return;
		
		LinkedList<V> stack = stepData.getStack();
		Map<V, BrandesVertexData<E> > vertex_data = stepData.getVertexData();
		Graph<V,E> graph = stepData.getGraph();
		Map<V, Double> dependency = new HashMap<V,Double>();
		
		for(V v : graph.getVertices())
			dependency.put(v, 0.0);
		V source = stack.getLast();
		
		for(V x : stack){
			
		    for (E e : vertex_data.get(x).incomingEdges)
		    {
		    	V w = graph.getOpposite(x, e);
		    	if(w == x)
		    		continue;
		        double partialDependency = (dependency.get(x))*vertex_data.get(w).numSPs/vertex_data.get(x).numSPs + vertex_data.get(w).numSPs; 
		        	//(vertex_data.get(w).numSPs) *	(1.0 + dependency.get(x));
		        
		        dependency.put(w, dependency.get(w)+partialDependency);
		    }
		    
		    if (x != source) 
		    {
		    	double x_score = currentOut.get(x).doubleValue();
		    	x_score += dependency.get(x);
		    	currentOut.put(x, x_score);
		    }
		}
		
	}

	@Override
	public Double getVertexScore(V v) {
		if(v == null)
			return null;
		return output.get(v);
	}

}
