package de.ipk_gatersleben.ag_nw.centilib.centralities;

import org.apache.commons.collections15.Transformer;

import edu.uci.ics.jung.algorithms.scoring.HITS;
import edu.uci.ics.jung.graph.Graph;

/**
 * @author Johannes Graessler
 */

/**
 * This is a wrapper class for the JUNG HITS, because the original HITS isn't 
 * a VertexScorer<V,Double>
 */
public class CentiLibHITS<V,E> extends VertexCentrality<V,E> {

	protected boolean authority;
	
	protected HITS<V,E> hits;
	
	
	public CentiLibHITS(Graph<V,E> graph, boolean useAuthority){
		this(graph,null,useAuthority);
/*		if(useAuthority)
			this.name += "-Auths";
		else
			this.name += "-Hubs";
		
		this.hits = new HITS<V,E>(graph);
		this.authority = useAuthority;
	*/}
	
	public CentiLibHITS(Graph<V,E> graph, Transformer<E, Double> edgeWeights, boolean useAuthority){
		super("HITS",graph);
		
		if(edgeWeights == null)
			this.hits = new HITS<V,E>(graph);
		else
			this.hits = new HITS<V,E>(graph,edgeWeights,0.0);
		
		if(useAuthority)
			this.name += "-Auths";
		else
			this.name += "-Hubs";
		this.authority = useAuthority;
	}

	@Override
	public Double getVertexScore(V v) {
		if(authority)
			return hits.getVertexScore(v).authority;
		return hits.getVertexScore(v).hub;
	}
	

	public int getIterations(){
		return hits.getIterations();
	}
	
	public void setMaxIterations(int iterations){
		hits.setMaxIterations(iterations);
	}
	
	public int getMaxIterations(){
		return hits.getMaxIterations();
	}
	
	public void setTolerance(double tolerance){
		hits.setTolerance(tolerance);
	}
	public void evaluate(){
		hits.evaluate();
		for(V v : graph.getVertices())
			output.put(v, getVertexScore(v));
	}
	
	private class WeightedHITS<V,E> extends HITS<V,E>{
		
		public WeightedHITS(Graph<V,E> graph, Transformer<E,Double> weights){
			super(graph,weights,0.0);
		}
		
		/**
		 * the edge weights are expected to be normalized
		 * so the vertex scores will be normalized too (see the documentation for HITS<V,E>
		 */
		@Override
		protected void normalizeScores(){}
	}
}
