package de.ipk_gatersleben.ag_nw.centilib.centralities;

import edu.uci.ics.jung.algorithms.shortestpath.Distance;
import edu.uci.ics.jung.algorithms.shortestpath.UnweightedShortestPath;
import edu.uci.ics.jung.graph.Graph;

/**
 * Closeness centrality.
 * 
 * @author Johannes Graessler
 * @author Dirk Koschuetzki
 */
public class Closeness<V,E> extends VertexCentrality<V,E> {

    public final static String KEY = "de.ipk_gatersleben.ag_nw.centilib.centralities.Closeness.RankScore";
    
    protected Distance<V> distance;

    public Closeness(Graph<V,E> graph) {
    	this(graph,new UnweightedShortestPath<V, E>(graph));
    }
    
    public Closeness(Graph<V,E> graph, Distance<V> distance){
    	super("Closeness",graph);
    	this.distance = distance;
    }

    public String getRankScoreKey() {
        return KEY;
    }

    public Double getVertexScore(V v) {

    	if(v == null) return null;
       	Double value = output.get(v);
	    if (value != null)
	    	if(value < 0)
	    		return null;
	    	else
	    		return value;

	    value = calculateClosenessCentrality(v);
        output.put(v, value);
        
        if(value < 0)
        	return null;
        else
        	return value;
    }

    private double calculateClosenessCentrality(V currentVertex) {

        double cl = 0;

        for (V target : graph.getVertices()) {
            Number currentDistance = distance.getDistance(currentVertex, target);
            if (currentDistance == null){
            	cl = Double.MAX_VALUE;
            	break;
            }
            else
            	cl += currentDistance.doubleValue();
        }
        if(cl == 0)
        	cl = Double.MAX_VALUE;
        assert (cl != 0);
        return 1 / cl;
    }
}