package de.ipk_gatersleben.ag_nw.centilib.centralities;

import org.apache.commons.collections15.Predicate;

import de.ipk_gatersleben.ag_nw.centilib.gui.actions.graphstatistics.WienerIndexAction;
import edu.uci.ics.jung.algorithms.filters.VertexPredicateFilter;
import edu.uci.ics.jung.algorithms.shortestpath.Distance;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computation of the closeness vitality centrality.
 *
 * @author Johannes Graessler
 * @author Dirk Koschuetzki
 * @author Michael Telgkamp
 */
public class ClosenessVitality<V,E> extends VertexCentrality<V,E> {

    public final static String KEY = "de.ipk_gatersleben.ag_nw.centilib.centralities.ClosenessVitality.RankScore";

    protected Distance<V> distance;
    

    
    public ClosenessVitality(Graph<V,E> graph, Distance<V> distance) {
    	super("ClosenessVitality",graph);
    	this.distance = distance;
    }

    public String getRankScoreKey() {
        return KEY;
    }

    public Double getVertexScore(V v) {
    	
    	if(output.get(v) != null)
    		return output.get(v);

        double wienerIndex = WienerIndexAction.calculateWienerIndex(graph,distance);

        if (new Double(wienerIndex).isInfinite()) {
            throw new IllegalArgumentException("Graph is not strong connected");
        }

        NotSamePredicate<V> filter = new NotSamePredicate<V>();
        
        VertexPredicateFilter<V, E> vpFilter = new VertexPredicateFilter<V, E>(filter);
        
        for(V w : graph.getVertices()){
        	filter.setVertex(w);
            double redWienerIndex = WienerIndexAction.calculateWienerIndex(vpFilter.transform(graph),distance);
            if (new Double(redWienerIndex).isInfinite()) {
                throw new IllegalArgumentException("Graph is not 2-strong connected");
            }
            output.put(w, wienerIndex - redWienerIndex);
        }
        return output.get(v);
    }

    protected class NotSamePredicate<T> implements Predicate<T>{
    	private T value;
    	
    	public void setVertex(T v){
    		value = v;
    	}
    	
    	public boolean evaluate(T v){
    		return !(value == v);
    	}
    }
}