package de.ipk_gatersleben.ag_nw.centilib.centralities;

import edu.uci.ics.jung.algorithms.shortestpath.Distance;
import edu.uci.ics.jung.algorithms.shortestpath.UnweightedShortestPath;
import edu.uci.ics.jung.graph.Graph;

/**
 * Eccentricity centrality.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 */
public class Eccentricity<V,E> extends VertexCentrality<V,E> {

    public final static String KEY = "de.ipk_gatersleben.ag_nw.centilib.centralities.Eccentricity.RankScore";
    
    protected Distance<V> distance;

    public Eccentricity(Graph<V,E> graph) {
    	this(graph,new UnweightedShortestPath<V, E>(graph));
    }

    public Eccentricity(Graph<V,E> graph, Distance<V> distance){
    	super("Eccentricity",graph);
    	this.distance = distance;
    }
    

	public String getRankScoreKey() {
        return KEY;
    }

    public Double getVertexScore(V v) {

    	if(output.get(v) != null)
	    		return output.get(v);

	    output.put(v, calculateEccenctricityCentrality(distance, v));

        return output.get(v);
    }

    private Double calculateEccenctricityCentrality(Distance<V> distance, V currentVertex) {

        double ecc = 0;

        for (V target : graph.getVertices()){
            Number newEcc = distance.getDistance(currentVertex, target);
            if (newEcc == null) {
				ecc = Double.POSITIVE_INFINITY;
				break;
            } else {
                ecc = (newEcc.doubleValue()) > ecc ? newEcc.doubleValue() : ecc;
            }
        }

        assert (ecc != 0);
        return 1 / ecc;
    }
}