package de.ipk_gatersleben.ag_nw.centilib.centralities;

import java.util.Map;

import org.apache.commons.collections15.BidiMap;

import cern.colt.matrix.DoubleMatrix1D;
import cern.colt.matrix.DoubleMatrix2D;
import cern.colt.matrix.linalg.EigenvalueDecomposition;
import edu.uci.ics.jung.algorithms.matrix.GraphMatrixOperations;
import edu.uci.ics.jung.algorithms.util.Indexer;
import edu.uci.ics.jung.graph.Graph;

/**
 * Eigenvector centrality.
 * 
 * @author Dirk Koschuetzki, Johannes Graessler
 */
public class Eigenvector<V,E> extends VertexCentrality<V,E>{
	
    protected Map<E, Number> edgeWeights = null;
    protected int maxIterations;
    
    public Eigenvector(Graph<V,E> graph) {
    	super("Eigenvector",graph);
    }

    public Eigenvector(Graph<V,E> graph, Map<E, Number> edgeWeights){
    	this(graph);
    	this.edgeWeights = edgeWeights;
    }
    
    @Override
    public Double getVertexScore(V v){
    	
    	if(output.get(v) != null)
    		return output.get(v);
    	
    		compute();
    		
    	return output.get(v);
    }
    protected void compute() {
        DoubleMatrix2D adjacencyMatrix = GraphMatrixOperations.graphToSparseMatrix(graph,edgeWeights);
        BidiMap<V,Integer> vertexIndexer = Indexer.create(graph.getVertices());

        // 
        //
        //
        // Beware: Colt works a little bit strange with the matrices!
        // Read the documentation!
        //
        //
        //

        EigenvalueDecomposition ed = new EigenvalueDecomposition(adjacencyMatrix);
        DoubleMatrix1D ev = extractRelevantVector(ed);

        for (V v : graph.getVertices()) {
            output.put(v, Math.abs(ev.get(vertexIndexer.get(v))));
        }
    }

    private DoubleMatrix1D extractRelevantVector(EigenvalueDecomposition ed) {
        int relevantPosition = -1;

        for (int i = 0; i < ed.getRealEigenvalues().size(); i++) {
            DoubleMatrix1D ev = ed.getV().viewColumn(i);

            int posNeg = 0;
            boolean allSameDirection = true;

            for (int j = 0; j < ev.size(); j++) {
                if (posNeg == 0) {
                    posNeg = (ev.get(j) >= 0.0d ? 1 : -1);
                } else {
                    if (posNeg == 1) {
                        if (ev.get(j) >= 0.0d) {
                            // ok
                        } else {
                            allSameDirection = false;
                            break;
                        }
                    } else {
                        if (ev.get(j) >= 0.0d) {
                            allSameDirection = false;
                            break;
                        } else {
                            // ok
                        }
                    }
                }
            }

            if (allSameDirection) {
                relevantPosition = i;
                break;
            }
        }
        assert (relevantPosition != -1);

        DoubleMatrix1D ev = ed.getV().viewColumn(relevantPosition);
        return ev;
    }
}