package de.ipk_gatersleben.ag_nw.centilib.centralities;

import java.util.LinkedList;
import java.util.List;

import org.apache.commons.collections15.BidiMap;

import edu.uci.ics.jung.algorithms.util.Indexer;
import edu.uci.ics.jung.graph.Graph;

/**
 * A ranker that uses the results from a loaded file.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 */
public class FileBasedRanker<V,E> extends VertexCentrality<V,E> {

	public final static String KEY = "de.ipk_gatersleben.ag_nw.centilib.centralities.FileBased.RankScore";


	public FileBasedRanker(String name, Graph<V,E> graph, LinkedList<? extends Number> rankings) {
		super(name,graph);
		setRankings(rankings);
	}

	public String getRankScoreKey() {
		return KEY;
	}

	protected void setRankings(List<? extends Number> rankings) {
		if(graph.getVertexCount() != rankings.size()){
			output = null;
			return;
		}
		
		BidiMap<V,Integer> indexer = Indexer.create(graph.getVertices());
		V currentVertex;
		int position = 0;
		for (Number rank : rankings){
			currentVertex = indexer.getKey(position);
			output.put(currentVertex, rank.doubleValue());
			position++;
		}
	}

	public Double getVertexScore(V v) {
		if(output == null)
			return Double.POSITIVE_INFINITY;
		return output.get(v);
	}
}