package de.ipk_gatersleben.ag_nw.centilib.centralities;

import edu.uci.ics.jung.algorithms.shortestpath.Distance;
import edu.uci.ics.jung.algorithms.shortestpath.UnweightedShortestPath;
import edu.uci.ics.jung.graph.Graph;

/**
 * Radiality Centrality
 *
 * @author Johannes Graessler 
 * @author Michael Telgkamp
 * @author Dirk Koschuetzki
 */
public class Radiality<V,E> extends VertexCentrality<V,E> {

    protected Distance<V> distance;
    
    protected Double diameter;
    
   
    
    public Radiality(Graph<V,E> graph, double diameter) {
        this(graph,new UnweightedShortestPath<V, E>(graph),diameter);
    }
    
    public Radiality(Graph<V,E> graph, Distance<V> distance, double diameter) throws IllegalArgumentException{
    	super("Radiality",graph);
    	this.distance = distance;
    	this.diameter = diameter; //CentralityHandler.getGraphCentralityValue(graph, CentralityHandler.DIAMETER,true);
    	//if(this.diameter == null)
    		//throw new IllegalArgumentException("The diameter could not be received!");
    }

    public Double getVertexScore(V v) {
    	
    	if(output.get(v) != null)
    		return output.get(v);
    	
    	double rad = 0.0;
        int numVertices = graph.getVertexCount();
        
        for (V w : graph.getVertices()) {
            Number uspValue = distance.getDistance(v, w);
            if (uspValue == null) {
                // Adding pos. Infinity once is enough. :-)
                rad = Double.POSITIVE_INFINITY ;
                break;
            } else {
                rad += (diameter + 1.0 - uspValue.doubleValue());
            }
        }
        
        output.put(v, rad / (numVertices - 1.0));
        return output.get(v);
    }
}