package de.ipk_gatersleben.ag_nw.centilib.centralities;

import org.apache.commons.collections15.BidiMap;

import cern.colt.matrix.DoubleFactory2D;
import cern.colt.matrix.DoubleMatrix1D;
import cern.colt.matrix.DoubleMatrix2D;
import cern.colt.matrix.linalg.EigenvalueDecomposition;
import cern.jet.math.Functions;
import edu.uci.ics.jung.algorithms.matrix.GraphMatrixOperations;
import edu.uci.ics.jung.algorithms.util.Indexer;
import edu.uci.ics.jung.graph.Graph;

//TODO: Rework the SALSA centrality!
/**
 * 
 * @author Johannes Graessler
 * @author Michael Telgkamp
 */
public class SALSA<V,E> extends VertexCentrality<V,E>{
	
    public final static String AUTHORITY_KEY = "de.ipk_gatersleben.ag_nw.centilib.centralities.SALSA.RankScore.AUTHORITY";

    public final static String HUB_KEY = "de.ipk_gatersleben.ag_nw.centilib.centralities.SALSA.RankScore.HUB";

    public final static boolean AUTHORITY = true;

    public final static boolean HUB = false;

    private boolean useAuthority;
    
    /**
     * Generates a SALSA Ranker
     * 
     * @param graph:
     *            The graph to handle with
     * @param version:
     *            either SALSA authority scores (use SALSA.AUTHORITY) or SALSA
     *            hub scores (use SALSA.HUB)
     */
    public SALSA(Graph<V,E> graph, boolean useAuthority) {
    	super("SALSA",graph);
    	this.useAuthority = useAuthority;
    }

    public String getRankScoreKey() {
        if (useAuthority) {
            return AUTHORITY_KEY;
        }
        return HUB_KEY;
    }

    public Double getVertexScore(V v) {
    	if(output.get(v) != null)
    		return output.get(v);

        DoubleMatrix2D adjacencyMatrix = GraphMatrixOperations.graphToSparseMatrix(graph, null);

        DoubleMatrix2D matrixP = computeMatrixP(adjacencyMatrix);

        DoubleMatrix2D matrixR = computeMatrixR(adjacencyMatrix);

        DoubleMatrix2D R = matrixR.copy();
        DoubleMatrix2D P = matrixP.copy();
        DoubleMatrix2D Pt = P.viewDice();
        DoubleMatrix2D dummy = null;
        DoubleMatrix2D AhSigma = R.zMult(Pt, dummy);

        R = null;
        Pt = null;
        R = matrixR.copy();
        DoubleMatrix2D Rt = R.viewDice();
        P = matrixP.copy();
        dummy = null;

        DoubleMatrix2D AaSigma = Rt.zMult(P, dummy);

        DoubleMatrix1D res;
        if (useAuthority) {
            res = new EigenvalueDecomposition(AaSigma).getRealEigenvalues();
        } else {
            res = new EigenvalueDecomposition(AhSigma).getRealEigenvalues();
        }

        BidiMap<V,Integer> vertexIndexer = Indexer.create(graph.getVertices());
        for (V w : graph.getVertices()) {
            int i = vertexIndexer.get(w);
            output.put(w, res.get(i));
        }
        return output.get(v);
    }

    private DoubleMatrix2D computeMatrixP(DoubleMatrix2D adjacencyMatrix) {
    	DoubleMatrix2D copy = adjacencyMatrix.copy();
    	return copy.assign(pWeightMatrix(adjacencyMatrix,copy), Functions.mult);
    }

    private DoubleMatrix2D pWeightMatrix(DoubleMatrix2D adjacencyMatrix, DoubleMatrix2D work) {
        DoubleMatrix2D columnFactor = DoubleFactory2D.dense.make(1, adjacencyMatrix.columns(), 0);
        for (int i = 0; i < work.columns(); i++) {
            double value = work.viewColumn(i).zSum();
            if (value == 0) {
                value = 1;
            } else {
                value = 1 / value;
            }
            columnFactor.set(0, i, value);
        }
        return DoubleFactory2D.dense.repeat(columnFactor, work.columns(), 1);
    }

    private DoubleMatrix2D computeMatrixR(DoubleMatrix2D adjacencyMatrix) {
        DoubleMatrix2D adjecencyMatrixCopy = adjacencyMatrix.copy();
        return adjecencyMatrixCopy.assign(rWeightMatrix(adjacencyMatrix,adjecencyMatrixCopy), Functions.mult);
    }

    private DoubleMatrix2D rWeightMatrix(DoubleMatrix2D adjacencyMatrix, DoubleMatrix2D work) {
        DoubleMatrix2D rowFactor = DoubleFactory2D.dense.make(adjacencyMatrix.rows(), 1, 0);
        for (int i = 0; i < work.rows(); i++) {
            double value = work.viewRow(i).zSum();
            if (value == 0) {
                value = 1;
            } else {
                value = 1 / value;
            }
            rowFactor.set(i, 0, value);
        }
        return DoubleFactory2D.dense.repeat(rowFactor, 1, work.rows());
    }
}