package de.ipk_gatersleben.ag_nw.centilib.centralities;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesSPData;
import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesSPListener;
import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesVertexData;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.UndirectedGraph;

public class SPBetweenness<V, E> extends VertexCentrality<V,E>
		implements	BrandesSPListener<V, E> {

	private Map<V, Double> currentOut = new HashMap<V,Double>();
	
	
	
	public SPBetweenness(Graph<V,E> graph) {
		super("Betweenness",graph);
		for(V v : graph.getVertices())
			currentOut.put(v, 0.0);
	}

	@Override
	public Double getVertexScore(V v) {
		if(v == null)
			return null;
		
		return output.get(v);
	}

	@Override //BrandesSP
	public void computeDependencies(BrandesSPData<V,E> stepData) {
		if(output.size() != 0 || stepData.getGraph() != graph)
			return;
		
		LinkedList<V> stack = stepData.getStack();
		Map<V, BrandesVertexData<E> > vertex_data = stepData.getVertexData();
		Map<V, Double> dependency = new HashMap<V,Double>();
		V source = stack.getLast();
		
		for(V v : graph.getVertices())
			dependency.put(v, 0.0);
		
		for(V target : stack){
			
		    for (E e : vertex_data.get(target).incomingEdges)
		    {
		    	V w = graph.getOpposite(target, e);
		        double partialDependency = 
		        	vertex_data.get(w).numSPs / vertex_data.get(target).numSPs *
		        	(1.0 + dependency.get(target));
		        
		        dependency.put(w, dependency.get(w)+partialDependency);
		    }
		    
		    if ( target != source) 
		    {
		    	double x_score = currentOut.get(target).doubleValue() + dependency.get(target);
		    	currentOut.put(target, x_score);
		    }
		}
	}

	@Override
	public void finished() {
		output  = currentOut;
		if(graph instanceof UndirectedGraph<?,?>)
			for(V v : graph.getVertices())
				output.put(v, output.get(v)/2);
	}

}
