package de.ipk_gatersleben.ag_nw.centilib.centralities;

import java.util.HashMap;
import java.util.Map;

import edu.uci.ics.jung.algorithms.shortestpath.Distance;
import edu.uci.ics.jung.graph.Graph;

/**
 *  A node importance ranker based on the Stress centrality.
 *
 * @author Johannes Graessler 
 * @author Michael Telgkamp
 * @author Dirk Koschuetzki
 */

// TODO: Rework this implementation, it is ok but VERY inefficient.
public class Stress<V,E> extends VertexCentrality<V,E> {

	
    public final static String KEY = "de.ipk_gatersleben.ag_nw.centilib.centralities.Stress.RankScore";

    protected Distance<V> distance;
    
    private Map<V, Map<V, Integer> > nrSTShortestPaths = new HashMap<V, Map<V, Integer> >();
    

    public Stress(Graph<V,E> graph, Distance<V> distance){
    	super("Stress",graph);
    	this.distance = distance;
    }

    public String getRankScoreKey() {
        return KEY;
    }

    public Double getVertexScore(V v) {
    	//TODO reimplement this algorithm, it's very slow -> maybe a good all-pair-shortestpath
    	if(output.get(v) != null)
    		return output.get(v);
    	long nrSP = 0;
    	for(V s : graph.getVertices())
    		output.put(s,0.0);
    	
    	for(V s : graph.getVertices()){
    		for(V t : graph.getVertices()){
    			if(t == s)
    				continue;
    			nrSP += nrSP(s,t);
    			for(V w : graph.getVertices()){
    				if(w == s || w == t)
    					continue;
    				if(isOnSTPath(s, w, t)){
    					output.put(w, output.get(w) + (nrSP(s,w).doubleValue()*nrSP(w,t).doubleValue()));
    				}
    			}
    		}
    	}
    	System.out.println("NrST-Paths: "+nrSP);
    	
    	return output.get(v);
    }
    private boolean isOnSTPath(V s, V v, V t) {
        double distST;
        double distSVandVT;
        try {
            distST = distance.getDistance(s, t).doubleValue();
            distSVandVT = distance.getDistance(s, v).doubleValue() + distance.getDistance(v, t).doubleValue();
        } catch (NullPointerException e) {
            return false;
        }
        return distST == distSVandVT;
    }
    
    private Integer nrSP(V s, V t){
		
    	if(graph.getPredecessors(t).contains(s))
    		return 1;
    	
    	Integer nrPaths =0;

    	if(nrSTShortestPaths.get(s) == null)
    		nrSTShortestPaths.put(s,new HashMap<V,Integer>());
    	else
    		if(nrSTShortestPaths.get(s).get(t) != null)
    			return nrSTShortestPaths.get(s).get(t);
    	
    	for(V v : graph.getPredecessors(t)){
    		if(isOnSTPath(s, v, t))
    			nrPaths += nrSP(s,v);
    	}
    	
    	nrSTShortestPaths.get(s).put(t, nrPaths);
     	return nrPaths;
    }
}