package de.ipk_gatersleben.ag_nw.centilib.centralities.parameter;


/**
 * An abstract class for centrality parameter values. The function askForValue()
 * is called by askUserForValue() and should be implemented to get the centrality specific
 * value from the user. Otherwise the method setValue(Double) should be used.
 * 
 * The value can be set only once;
 * 
 * Use getMinValue() and getMaxValue() to get the range of possible numbers
 * 
 * @author Johannes Graessler 
 */

public abstract class CentralityParameter {

	protected Double param = null;
	
	/**
	 * This method should be called to get the parameter from the user. 
	 * @return the parameter value.
	 */
	public boolean askUserForValue(){
		if(param != null)
			return true;
		
		Double temp = askForValue();
		return(setValue(temp));
	}
	
	public boolean setValue(Double value){
		if(param == null && value>=getMinValue() && value <= getMaxValue())
			param = value;
		else
			return false;
		return true;
	}
	
	public Double getValue(){
		return param;		
	}
	
	protected abstract Double askForValue();
	public abstract Double getMinValue();
	public abstract Double getMaxValue();
}
