package de.ipk_gatersleben.ag_nw.centilib.centralities.parameter;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.centralities.HubbellIndex;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;
import edu.uci.ics.jung.graph.Graph;

public class HubbellParameter<V,E> extends CentralityParameter {

	private CentiLib<V,E> centilib;
	
	private Graph<V,E> graph;
	
	public HubbellParameter(CentiLib<V,E> centilib, Graph<V,E> graph){
		super();
		this.centilib = centilib;
		this.graph = graph;
	}
	
	@Override
	public Double getMinValue(){
		return 0.0;
	}
	
	@Override
	public Double getMaxValue(){
		return Double.MAX_VALUE;
	}
	
	@Override
	public boolean setValue(Double value){
		if(param != null)
			return false;
		
		if(HubbellIndex.isSolvable(graph,GraphCachingUtils.getEdgeWeights(graph), value))
			param = value;
		else
			return false;
		
		return true;
	}
	
	@Override
	protected Double askForValue() {

		if(centilib == null || graph == null)
			return null;
		
        Double weightFactor = null;
        boolean isSolvable = false;
        
        while(!isSolvable){
        	
        	weightFactor = centilib.askForDouble("HubbelIndex for graph: "+
        			GraphCachingUtils.getGraphName(graph)+"\n\nPlease enter a weight factor.", "0.5", 0.0,
        		Double.MAX_VALUE);
        	if(weightFactor == null)
        		return null;
        	
        	isSolvable = setValue(weightFactor);
        	if(!isSolvable){
        		if(!centilib.askForBoolean("HubbelIndex is not computable for graph '"+
        				GraphCachingUtils.getGraphName(graph)+"'\n" +
        				"with the given weight factor." +
        				"\n\nDo you want to try another weight factor?")){
        			isSolvable = true;
        			weightFactor = null;
        		}
        	}
        }

		return weightFactor;
	}

}
