package de.ipk_gatersleben.ag_nw.centilib.centralities.parameter;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.centralities.KatzStatusIndex;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;
import edu.uci.ics.jung.graph.Graph;

public class KatzStatusParameter<V,E> extends CentralityParameter {

	private CentiLib<?,?> centilib;
	
	private Graph<V,E> graph;
	
	private Double maxAlpha = null;
	
	public KatzStatusParameter(CentiLib<?,?> centilib, Graph<V,E> graphName){
		
		this.centilib = centilib;
		this.graph= graphName;
	}
	
	@Override
	public Double getMinValue(){
		return 0.00000001;
	}
	
	@Override
	public Double getMaxValue(){
		if(graph == null)
			return null;
		if(maxAlpha == null)
			maxAlpha = KatzStatusIndex.getMaxAlphaValue(graph);
		
		return maxAlpha-0.00000001;
	}
	
	@Override
	protected Double askForValue() {
		String graphName = GraphCachingUtils.getGraphName(graph);
		if(centilib == null || graph == null)
			return null;
		
		if(maxAlpha == null)
			maxAlpha = KatzStatusIndex.getMaxAlphaValue(graph);

        Double alpha = 0.0;
        
        while(alpha != null &&(alpha == 0.0 || alpha == maxAlpha)){
        	alpha = centilib.askForDouble("Katz Status Index: Please enter an alpha value for Graph '"+graphName+"!", Double.toString(maxAlpha / 2),
                0, maxAlpha);
        	if(alpha != null && (alpha == 0 || alpha == maxAlpha)){
        		centilib.showInformationDialog("error", "The alpha value has to be bigger then 0 and smaller then "+maxAlpha+".");
        		alpha = null;
        	}
        }
        
        return alpha;	
	}	
}
