package de.ipk_gatersleben.ag_nw.centilib.graphstatistics;

import edu.uci.ics.jung.algorithms.shortestpath.Distance;
import edu.uci.ics.jung.algorithms.shortestpath.UnweightedShortestPath;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computes the Wiener index of the graph.
 *
 * @author Johannes Graessler 
 * @author Michael Telgkamp
 * @author Dirk Koschuetzki
 */
public class WienerIndex<V,E> {

    private double wienerIndex = Double.NaN;

    private Graph<V,E> graph = null;
    
    private Distance<V> distance;

    private boolean isCalculated = false;

    public WienerIndex(Graph<V,E> graph){
    	this(graph,new UnweightedShortestPath<V, E>(graph));
    }
    public WienerIndex(Graph<V,E> graph,Distance<V> distance) {
        this.graph = graph;
        this.distance = distance;
        isCalculated = false;
    }

    public double getWienerIndex() {
        if (!isCalculated) {
            computeWienerIndex();
        }
        return wienerIndex;
    }

    private void computeWienerIndex() {
        wienerIndex = 0;

        for (V sourceVertex : graph.getVertices()) {
            for (V targetVertex : graph.getVertices()) {

            	Number sourceTargetDistance = distance.getDistance(sourceVertex, targetVertex);

                if (sourceTargetDistance == null) {
                    // If we add infinity once, then its ok. :-)
                    wienerIndex += Double.POSITIVE_INFINITY;
                    isCalculated = true;
                    return;
                } else {
                    wienerIndex += sourceTargetDistance.doubleValue();
                }
            }
        }

        isCalculated = true;
    }
}
