package de.ipk_gatersleben.ag_nw.centilib.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.TableModelEvent;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import org.apache.commons.collections15.Transformer;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.AssistancePanelAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.ComputeCentralitiesAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.graphstatistics.ComputeGraphStatisticsAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.io.SaveCentralityTableAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.io.SaveCentralityVectorAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.io.SaveMultipleCentralityTablesAction;
import de.ipk_gatersleben.ag_nw.centilib.plugin.ApplicationInterface;
import de.ipk_gatersleben.ag_nw.centilib.utils.CentralityHandler;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;
import edu.uci.ics.jung.graph.Graph;



@SuppressWarnings("serial")
public class AssistancePanel<V,E> extends JPanel implements ActionListener {
	
	protected static final String SELECTACTIONSTATUS = "Select an action..";
	
	protected static final String COMPUTECENTSSTATUS = "Compute centralities";
	
	protected static final String GRAPHSTATSSTATUS = "Compute graph statistics";
	
	protected static final String EXPORTITEM = "Export:";
	
	protected static final String EXPORTVECTORSTATUS = "  each centrality to a vector"; 
	
	protected static final String EXPORTMULTFILESSTATUS = "  to a table for each graph (multiple files)";
	
	protected static final String EXPORTFILESTATUS = "  to one table for all graphs (one file)";
	
	protected static final String ADDATTRIBUTESSTATUS = "Add as attributes";
	
	private JLabel graphLabel = new JLabel("Available Graphs");
	
	private JLabel centLabel = new JLabel("Available Centralities");
	
	private JTable graphTable = new JTable();
	
	private JTable centTable = new JTable();
	

	private ProgressPanel progressPanel = new ProgressPanel();
	
	private JComboBox actionsBox = new JComboBox();
	
	private JButton startButton = new JButton("Perform Action");
	
	private ListTableModel<Graph<V,E>> graphTableModel;

	private ListTableModel<String> centTableModel;
	
	private Transformer<String,String> toString;
	
	private Transformer<Graph<V,E>, String> graphToString =  
		new Transformer<Graph<V, E>, String>() {
					public String transform(Graph<V, E> graph) {
						return GraphCachingUtils.getGraphName(graph);
					}
				};
 
	private CentiLib<V,E> centilib;

	private ApplicationInterface<V, E> application;
	
	public AssistancePanel(CentiLib<V,E> centilib) {
		super();	
		
		this.centilib = centilib;
		this.application = centilib.getAppInterface();
		
		graphTable.setToolTipText("Press and hold 'Ctrl' to select multiple graphs.");
		startButton.addActionListener(this);
		actionsBox.addActionListener(this);
		actionsBox.addItem(SELECTACTIONSTATUS);
		actionsBox.addItem(new ComputeCentralitiesAction<V, E>(centilib,COMPUTECENTSSTATUS));
		actionsBox.addItem(new ComputeGraphStatisticsAction<V, E>(centilib, GRAPHSTATSSTATUS));
		actionsBox.addItem(ADDATTRIBUTESSTATUS);
		actionsBox.addItem(EXPORTITEM);
		actionsBox.addItem(new SaveCentralityVectorAction<V, E>(centilib, EXPORTVECTORSTATUS));
		actionsBox.addItem(new SaveMultipleCentralityTablesAction<V, E>(centilib, EXPORTMULTFILESSTATUS));
		actionsBox.addItem(new SaveCentralityTableAction<V, E>(centilib, EXPORTFILESTATUS));
		
		toString = new Transformer<String,String>(){
			public String transform(String s){
				return (String) s;
			}
		};
		
		centTableModel = null;
		//centTable.setModel(new DefaultTableModel());
		//centTable.tableChanged(new TableModelEvent(centTableModel));
		//setBackground(Color.BLUE);
        setBorder(BorderFactory.createLineBorder(Color.BLACK, 1));
        setPreferredSize(new Dimension(150, 500));
        setLayout(new GridBagLayout());

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridy = GridBagConstraints.RELATIVE;
        gbc.gridx = 0;

        gbc.weightx = 0.0;
        gbc.weighty = 0.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        add(graphLabel, gbc);

        gbc.weightx = 0.75;
        gbc.weighty = 0.75;
        gbc.fill = GridBagConstraints.BOTH;
        add(new JScrollPane(graphTable), gbc);
        
        gbc.weightx = 0.0;
        gbc.weighty = 0.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        add(actionsBox,gbc);
        add(new JLabel("  "),gbc);
        add(centLabel, gbc);
        
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.BOTH;
        add(new JScrollPane(centTable), gbc);
        
        gbc.weightx = 0.0;
        gbc.weighty = 0.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        add(startButton,gbc);
        add(progressPanel,gbc);
        
        validate();
        
	}

	public void updateGraphs(){
		
		Collection<Graph<V,E> > graphs = GraphCachingUtils.getGraphs();
		
		if(graphs == null || (graphs.size() == 0 && graphTableModel == null))
			return;
		
		TableModel t;
		
		if(graphs == null || graphs.size() == 0){
			t = new DefaultTableModel();
			graphTableModel = null;
		}
		else{
			graphTableModel = new ListTableModel<Graph<V, E>>(graphs,
				"Available Graphs",graphToString);
			t = graphTableModel;
		}
		
		graphTable.setModel(t);
		graphTable.tableChanged(new TableModelEvent(t));
		
		if(graphTableModel != null && graphTableModel.getRowCount() > 0){
			Graph<V,E> currentGraph = centilib.getCurrentGraph();
			for(int i=0; i<graphTableModel.getRowCount();i++)
				if(currentGraph == graphTableModel.getItemAt(i, 0))
					graphTable.changeSelection(i, 0, false, false);
		}
		
		if(centTableModel != null)
			centTable.changeSelection(centTable.getSelectedRow(), 0, false, false);
		else setCentralityTable(null);
	}

	private void showAvailableCentsForGraphs(){
		
		Collection<Graph<V,E> > graphs = getSelectedGraphs();
		if(graphs == null) return;
		
		Set<String> scorers = new HashSet<String>();
		
		for(Graph<V,E> g : graphs){
			scorers.addAll(CentralityHandler.getAvailableCentNames(g));
		}
		
		setCentralityTable(scorers);
	}
	
	private void setCentralityTable(Collection<String> cents){
		
		TableModel t;
		if(cents == null || cents.size()==0){
			centTableModel = null;
			t = new DefaultTableModel();
			//actionsBox.setSelectedIndex(0);
		}
		else{
			centTableModel = new ListTableModel<String>(cents,"Available centralities", toString);
			t = centTableModel;
		}
		
		centTable.setModel(t);
		centTable.tableChanged(new TableModelEvent(t));
		centTable.validate();
	}
	
	public Collection<Graph<V,E> > getSelectedGraphs(){
		
		if(graphTableModel == null)
			return null;
		
		Collection<Graph<V,E> > graphs = new ArrayList<Graph<V,E> >();
		int min = ((ListSelectionModel) graphTable.getSelectionModel()).getMinSelectionIndex();
		int max = ((ListSelectionModel) graphTable.getSelectionModel()).getMaxSelectionIndex();
		
		for(int i=min; i<=max; i++)
			if(graphTable.getSelectionModel().isSelectedIndex(i))
				if(graphTableModel.getItemAt(i,0) != null && graphTableModel.getItemAt(i,0) instanceof Graph<?,?>)
					graphs.add(graphTableModel.getItemAt(i,1));
		
		return graphs;
	}
	
	public List<String> getSelectedCents(){
		if(centTableModel == null)
			return null;
		
		List<String> cents = new ArrayList<String>();
		int min = ((ListSelectionModel) centTable.getSelectionModel()).getMinSelectionIndex();
		int max = ((ListSelectionModel) centTable.getSelectionModel()).getMaxSelectionIndex();
		
		for(int i=min;i<=max;i++)
			if( ((ListSelectionModel)centTable.getSelectionModel()).isSelectedIndex(i))
				if(centTableModel.getItemAt(i, 0) != null)
					cents.add(centTableModel.getItemAt(i, 0));
		
		return cents;
	}

	@SuppressWarnings("unchecked")
	@Override //ActionListener -> actionsBox
	public void actionPerformed(ActionEvent e) {
		
		// if the user didn't do anything
		//if(e.getModifiers() == 0)
			//return;
		
		// if we are resetting
		if(actionsBox.getSelectedItem() == SELECTACTIONSTATUS)
			return;

		Collection<Graph<V,E> > graphs = getSelectedGraphs();
		List<String> cents = getSelectedCents();
		
		if(graphs == null || graphs.size() == 0){
			centilib.showInformationDialog("Hint", "Please select at least one graph");
			actionsBox.setSelectedIndex(0);
			return;
		}
		
		Object item = actionsBox.getSelectedItem();
		
		// ******* actionsBoxAction
		if(e.getSource() == actionsBox){

			if(item == EXPORTITEM){
				//actionsBox.setSelectedIndex(0);
			}
			
			if(item == ADDATTRIBUTESSTATUS)
				showAvailableCentsForGraphs();
			
			if(item instanceof AssistancePanelAction<?, ?>)
				setCentralityTable(((AssistancePanelAction) item).getPossibleCentralities(graphs));
			
			return;
		}
		
		// ******* startButtonAction
		if(e.getSource() == startButton){
			
			if(item == SELECTACTIONSTATUS){
				centilib.showInformationDialog("Hint", "Please select an action first");
				return;
			}
			
			if(cents == null || cents.size() == 0){
				centilib.showInformationDialog("Error", "Please select at least one centrality");
				return;
			}
			
			if(item == ADDATTRIBUTESSTATUS){
				application.addAttributes(graphs, cents);
				
				centilib.showInformationDialog("Finished", "Finished adding the attributes.");
			}

			if(item instanceof AssistancePanelAction<?,?>)
				((AssistancePanelAction<V,E>) item).run(graphs, cents);
		}

	}
	
    public void setOperationString(String s) {
        progressPanel.setOperationString(s);
    }

    public void setProgressString(String s) {
        progressPanel.setProgressString(s);
    }

    public void startProgressBar() {
        progressPanel.startProgressBar();
    }	
    
    public void stopProgressBar(){
    	progressPanel.stopProgressBar();
    }
}
