package de.ipk_gatersleben.ag_nw.centilib.gui;

import javax.swing.JMenu;
import javax.swing.MenuElement;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;

/**
 * The menu listener that enable/disables the centralities depending on the type of the graph.
 *
 * @author Johannes Graessler
 * @author Dirk Koschuetzki
 */
public class CentralityMenuListener implements MenuListener {

    private CentiLib<?,?> centilib;

    private JMenu directedMenu;

    private JMenu undirectedMenu;

    public CentralityMenuListener(CentiLib<?,?> centilib, JMenu directedMenu, JMenu undirectedMenu) {
        this.centilib = centilib;
        this.directedMenu = directedMenu;
        this.undirectedMenu = undirectedMenu;
    }

    public void menuDeselected(MenuEvent evt) {
        // Not used
    }

    public void menuCanceled(MenuEvent evt) {
        // Not used
    }

    public void menuSelected(MenuEvent evt) {
        boolean isUndirected;
        boolean isDirected;

        if (centilib.getCurrentGraph() == null || centilib.getCurrentGraph().getVertexCount() == 0) {
            isUndirected = false;
            isDirected = false;
        }
        else{
        	isUndirected = GraphCachingUtils.isUndirected(centilib.getCurrentGraph());
            isDirected = GraphCachingUtils.isDirected(centilib.getCurrentGraph());
        }

        MenuElement[] directedMenuElements = directedMenu.getSubElements()[0].getSubElements();
        for (int i = 0; i < directedMenuElements.length; i++) {
            directedMenuElements[i].getComponent().setEnabled(isDirected);
        }

        MenuElement[] undirectedMenuElements = undirectedMenu.getSubElements()[0].getSubElements();
        for (int i = 0; i < undirectedMenuElements.length; i++) {
            undirectedMenuElements[i].getComponent().setEnabled(isUndirected);
        }
    }

}