package de.ipk_gatersleben.ag_nw.centilib.gui;

import java.awt.HeadlessException;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.filechooser.FileFilter;

import de.ipk_gatersleben.ag_nw.centilib.utils.CentiLibFileWrapper;

/**
 * A file chooser that filters based on supported file extensions.
 *
 * @author Johannes Graessler
 * @author Dirk Koschuetzki
 * @author Michael Telgkamp
 */
public class GraphFileChooser extends JFileChooser {

    private static GraphFileChooser theInstance;

    private JFrame centilibTopFrame = null;

    private CentiLibMultiFF allReadableGraphFF;

    private CentiLibMultiFF allSaveableGraphFF;

    private CentiLibMultiFF allReadableVectorFF;

    private CentiLibMultiFF allWriteableVectorFF;

    private CentiLibMultiFF allWriteableSelectionFF;

    private CentiLibSingleFF tableFF;
    
    private CentiLibSingleFF pajekNetFF;

    private CentiLibSingleFF graphMLFF;

    private CentiLibSingleFF adjMatrixFF;

    private CentiLibSingleFF xwgFF;

    private CentiLibSingleFF dotFF;

    private CentiLibSingleFF dipFF;

    private CentiLibSingleFF bindFF;

    private CentiLibSingleFF pajekVectorFF;

    private CentiLibSingleFF tsvVectorFF;

    private GraphFileChooser(JFrame centilibTopFrame) {
        super();
        this.centilibTopFrame = centilibTopFrame;
        setMultiSelectionEnabled(false);
        this.setAcceptAllFileFilterUsed(false);
        initFFs();
    }

    public static void init(JFrame centilibTopFrame) {
        theInstance = new GraphFileChooser(centilibTopFrame);
    }

    private static synchronized GraphFileChooser getInstance() {
        assert (theInstance != null);
        return theInstance;
    }

    public static CentiLibFileWrapper showOpenNetworkDialog() throws HeadlessException {
        GraphFileChooser inst = getInstance();

        inst.removeFilters();
        inst.addLoadNetFilters();
        inst.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);

        int selectResult = inst.showOpenDialog(inst.getCentiBiNTopFrame());
        if (selectResult == JFileChooser.APPROVE_OPTION) {
            inst.setCurrentDirectory(inst.getSelectedFile());
            return new CentiLibFileWrapper(inst.getSelectedFile());
        } else {
            return null;
        }
    }

    public static CentiLibFileWrapper showOpenVectorDialog() throws HeadlessException {
        GraphFileChooser inst = getInstance();

        inst.removeFilters();
        inst.addReadableVectorFilters();
        inst.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);

        int selectResult = inst.showOpenDialog(inst.getCentiBiNTopFrame());
        if (selectResult == JFileChooser.APPROVE_OPTION) {
            inst.setCurrentDirectory(inst.getSelectedFile());
            return new CentiLibFileWrapper(inst.getSelectedFile());
        } else {
            return null;
        }
    }

    public static CentiLibFileWrapper showSelectDirectoryDialog() throws HeadlessException {
        GraphFileChooser inst = getInstance();

        inst.removeFilters();
        inst.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

        int selectResult = inst.showDialog(inst.getCentiBiNTopFrame(), "Select");
        if (selectResult == JFileChooser.APPROVE_OPTION) {
            inst.setCurrentDirectory(inst.getSelectedFile());
            return new CentiLibFileWrapper(inst.getSelectedFile());
        } else {
            return null;
        }
    }

    public static CentiLibFileWrapper showSaveNetworkDialog(String recommendedName) throws HeadlessException {
        GraphFileChooser inst = getInstance();

        
        inst.removeFilters();
        inst.addSaveNetFilters();
        inst.setSelectedFile(new File(inst.getCurrentDirectory(), recommendedName));
        inst.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);

        int selectResult = inst.showSaveDialog(inst.getCentiBiNTopFrame());
        if (selectResult == JFileChooser.APPROVE_OPTION) {
            inst.setCurrentDirectory(inst.getSelectedFile());
            return new CentiLibFileWrapper(inst.getSelectedFile());
        } else {
            return null;
        }
    }

    public static CentiLibFileWrapper showSaveVectorDialog(String recommendedName) throws HeadlessException {
        GraphFileChooser inst = getInstance();

        inst.removeFilters();
        inst.addWriteableVectorFilters();
        recommendedName = getNonExistingDefaultFileName(recommendedName);
        File f = new File(inst.getCurrentDirectory(), recommendedName);

        inst.setSelectedFile(f);
        inst.setFileSelectionMode(JFileChooser.FILES_ONLY);//_AND_DIRECTORIES);
        
        int selectResult = inst.showSaveDialog(inst.getCentiBiNTopFrame());
        if (selectResult == JFileChooser.APPROVE_OPTION) {
        	validateFileExtension();
            inst.setCurrentDirectory(inst.getSelectedFile());
            return new CentiLibFileWrapper(inst.getSelectedFile());
        } else {
            return null;
        }
    }

    public static CentiLibFileWrapper showSaveSelectionDialog(String recommendedName) throws HeadlessException {
        GraphFileChooser inst = getInstance();
        
        inst.removeFilters();
        inst.addWriteableSelectionFilters();
        recommendedName  = getNonExistingDefaultFileName(recommendedName);
        File f = new File(inst.getCurrentDirectory(), recommendedName);
        
        inst.setSelectedFile(f);
        inst.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);

        int selectResult = inst.showSaveDialog(inst.getCentiBiNTopFrame());
        if (selectResult == JFileChooser.APPROVE_OPTION) {
            inst.setCurrentDirectory(inst.getSelectedFile());
            return new CentiLibFileWrapper(inst.getSelectedFile());
        } else {
            return null;
        }
    }
    
    public static CentiLibFileWrapper showSaveTableDialog(String recommendedName) throws HeadlessException {
    	GraphFileChooser inst = getInstance();

        inst.removeFilters();
        inst.addWritableTableFilters();
    	recommendedName = getNonExistingDefaultFileName(recommendedName);

    	File f = new File(inst.getCurrentDirectory(), recommendedName);
        inst.setSelectedFile(f);
        inst.setFileSelectionMode(JFileChooser.FILES_ONLY);//_AND_DIRECTORIES);
        

        int selectResult = inst.showSaveDialog(inst.getCentiBiNTopFrame());
        if (selectResult == JFileChooser.APPROVE_OPTION) {
        	validateFileExtension();
        	inst.setCurrentDirectory(inst.getSelectedFile());
            return new CentiLibFileWrapper(inst.getSelectedFile());
        } else {
            return null;
        }
    }
    
    /**
     * Adds the selected suffix to the selected file name.
     */
    protected static void validateFileExtension(){
    	GraphFileChooser inst = getInstance();
    	String absolutePath = inst.getSelectedFile().getAbsolutePath();
    	
    	absolutePath = validateFileExtension(absolutePath);
    	inst.setSelectedFile(new File(absolutePath));
    }
    
    /**
     * Adds the selected suffix to the given file name.
     */
    protected static String validateFileExtension(String fileName){
    	GraphFileChooser inst = getInstance();
    	
    	if(fileName.charAt(fileName.length()-4) == '.')
    		return fileName;
    	
    	return fileName+((CentiLibFF) inst.getFileFilter()).getSuffix();
    }
    
    /**
     * Returns a file name which does not exist in the current directory. Therefore files with equal names are numbered consecutively starting from 1.
     * The file filter for this instance has to be set before using this method.
     * 
     * @param recommendedName The file name to use
     * @return a non existing file name
     */
    private static String getNonExistingDefaultFileName(String recommendedName){
    	
    	String destFileName = validateFileExtension(recommendedName);
    	File destFile = new File(getInstance().getCurrentDirectory(),destFileName);
    	String nameWithoutExtension = "";
    	int counter = 1;
    	
    	if(recommendedName.charAt(recommendedName.length()-4) == '.')
    		nameWithoutExtension = recommendedName.substring(0, recommendedName.length()-4);
    	else
    		nameWithoutExtension = recommendedName;
    	
    	while(destFile.exists()){
    		destFileName = validateFileExtension(nameWithoutExtension+"("+counter+")");
    			destFile = new File(getInstance().getCurrentDirectory(),destFileName);
    			counter++;
    	}
    	
    	return destFileName.substring(0,destFileName.length()-4);
    }
    
    // /////// FF handling starts here


    private void initFFs() {
        if (true) {
            allReadableGraphFF = new CentiLibMultiFF(new String[] { ".net", ".mat", ".xml", ".tab" },
                    "All readable files (*.net, *.mat, *.xml, *.tab)");
            allSaveableGraphFF = new CentiLibMultiFF(new String[] { ".net", ".mat", ".xwg", ".dot" },
                    "Choose format from suffix (*.net, *.mat, *.xwg, *.dot)");
            allReadableVectorFF = new CentiLibMultiFF(new String[] { ".vec" }, "Choose format from suffix (*.vec)");
            
            //TODO .tsv (TSVFiler) ermoeglichen -> fuer export schon moeglich, aber kein import
            allWriteableVectorFF = new CentiLibMultiFF(new String[] { ".vec", ".tsv" },
                    "Choose format from suffix (*.vec, *.tsv)");
            allWriteableSelectionFF = new CentiLibMultiFF(new String[] { ".vec" }, "Choose format from suffix (*.vec)");
            tableFF = new CentiLibSingleFF(".tsv","TSV Table  files (*.tsv)");
            pajekNetFF = new CentiLibSingleFF(".net", "Pajek Net files (*.net)");
            graphMLFF = new CentiLibSingleFF(".xml", "GraphML files (*.xml)");
            adjMatrixFF = new CentiLibSingleFF(".mat", "Adjacency Matrix files (*.mat)");
            xwgFF = new CentiLibSingleFF(".xwg", "XWG files (*.xwg)");
            dotFF = new CentiLibSingleFF(".dot", "DOT files (*.dot)");
            dipFF = new CentiLibSingleFF(".tab", "DIP files in tab format (*.tab)");
            bindFF = new CentiLibSingleFF(".xml", "BIND files in xml format (*.xml)");
            pajekVectorFF = new CentiLibSingleFF(".vec", "Pajek Vector files (*.vec)");
            tsvVectorFF = new CentiLibSingleFF(".tsv", "TSV Vector files (*.tsv)");
        } else {
            allReadableGraphFF = new CentiLibMultiFF(new String[] { ".net", ".mat", ".xml", ".tab" },
                    "All readable files (*.net, *.mat, *.xml, *.tab)");
            allSaveableGraphFF = new CentiLibMultiFF(new String[] { ".net", ".mat", ".dot" },
                    "Choose format from suffix (*.net, *.mat, *.dot)");
            allReadableVectorFF = new CentiLibMultiFF(new String[] { ".vec" }, "Choose format from suffix (*.vec)");
            allWriteableVectorFF = new CentiLibMultiFF(new String[] { ".vec", ".tsv" },
                    "Choose format from suffix (*.vec, *.tsv)");
            allWriteableSelectionFF = new CentiLibMultiFF(new String[] { ".vec" }, "Choose format from suffix (*.vec)");

            pajekNetFF = new CentiLibSingleFF(".net", "Pajek Net files (*.net)");
            graphMLFF = new CentiLibSingleFF(".xml", "GraphML files (*.xml)");
            adjMatrixFF = new CentiLibSingleFF(".mat", "Adjacency Matrix files (*.mat)");
            xwgFF = null;
            dotFF = new CentiLibSingleFF(".dot", "DOT files (*.dot)");
            dipFF = new CentiLibSingleFF(".tab", "DIP files in tab format (*.tab)");
            bindFF = null;

            pajekVectorFF = new CentiLibSingleFF(".vec", "Pajek Vector files (*.vec)");
            tsvVectorFF = new CentiLibSingleFF(".tsv", "TSV Vector files (*.tsv)");
        }
    }

    private void removeFilters() {
        resetChoosableFileFilters();
        // Removes the "all" filter.
        //if(this.getChoosableFileFilters().length >= 1)
        	//removeChoosableFileFilter(this.getChoosableFileFilters()[0]);
    }

    private void addLoadNetFilters() {
        addChoosableFileFilter(allReadableGraphFF);
        addChoosableFileFilter(graphMLFF);
        addChoosableFileFilter(adjMatrixFF);
        addChoosableFileFilter(dipFF);
        addChoosableFileFilter(bindFF);
        addChoosableFileFilter(pajekNetFF);
        setFileFilter(allReadableGraphFF);
    }

    private void addSaveNetFilters() {
        addChoosableFileFilter(allSaveableGraphFF);
        addChoosableFileFilter(adjMatrixFF);
        addChoosableFileFilter(pajekNetFF);
        addChoosableFileFilter(dotFF);
        if (xwgFF != null) {
            addChoosableFileFilter(xwgFF);
        }
        setFileFilter(allSaveableGraphFF);
    }

    private void addReadableVectorFilters() {
        addChoosableFileFilter(allReadableVectorFF);
        addChoosableFileFilter(pajekVectorFF);
        setFileFilter(allReadableVectorFF);
    }

    private void addWriteableVectorFilters() {
        addChoosableFileFilter(allWriteableVectorFF);
        addChoosableFileFilter(tsvVectorFF);
        addChoosableFileFilter(pajekVectorFF);
        setFileFilter(allWriteableVectorFF);
    }
    
    private void addWritableTableFilters(){
    	addChoosableFileFilter(tsvVectorFF);
    	setFileFilter(tsvVectorFF);
    }

    private void addWriteableSelectionFilters() {
        addChoosableFileFilter(allWriteableSelectionFF);
        addChoosableFileFilter(pajekVectorFF);
        setFileFilter(allWriteableSelectionFF);
    }

    private abstract class CentiLibFF extends FileFilter{
    	public abstract String getSuffix();
    }
    
    private class CentiLibSingleFF extends CentiLibFF{
        private String suffix;

        private String desc;

        public CentiLibSingleFF(String suffix, String desc) {
            this.suffix = suffix;
            this.desc = desc;
        }
        
        public String getSuffix(){
        	return suffix;
        }

        public boolean accept(File pathname) {
            return pathname.isDirectory() || pathname.getName().toLowerCase().endsWith(suffix);
        }

        public String getDescription() {
            return desc;
        }
    }

    /**
     * A FileFilter for multiple file types. The first suffix will be used as default.
     */
    private class CentiLibMultiFF extends CentiLibFF{
        private String[] suffices;

        private String desc;

        public CentiLibMultiFF(String suffices[], String desc) {
            this.suffices = suffices;
            this.desc = desc;
        }

        public String getSuffix(){
        	return suffices[0];
        }
        public boolean accept(File pathname) {
            if (pathname.isDirectory()) {
                return true;
            }

            for (int i = 0; i < suffices.length; i++) {
                String suffix = suffices[i];
                if (pathname.getName().toLowerCase().endsWith(suffix)) {
                    return true;
                }
            }
            return false;
        }

        public String getDescription() {
            return desc;
        }
    }

    /**
     * @return Returns the centilibTopFrame.
     */
    private JFrame getCentiBiNTopFrame() {
        return centilibTopFrame;
    }
}