package de.ipk_gatersleben.ag_nw.centilib.gui;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.border.LineBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.ConnectedComponents;
import de.ipk_gatersleben.ag_nw.centilib.plugin.ApplicationInterface;
import de.ipk_gatersleben.ag_nw.centilib.plugin.DefaultEdgeAttribute;
import de.ipk_gatersleben.ag_nw.centilib.plugin.DoubleAttribute;
import de.ipk_gatersleben.ag_nw.centilib.utils.CentralityHandler;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.util.EdgeType;

@SuppressWarnings("serial")
public class GraphStatisticsPanel<V,E> extends JPanel implements ListSelectionListener, ActionListener{

	private JTable compTable = new JTable();
	
	private ConnectedComponentTableModel<V> compTableModel;
	
	private JComboBox attributesComboBox = new JComboBox();
	
	private JComboBox connectedComboBox = new JComboBox(new String[] {"--Select an item--","Connected Components","Strongly Connected Components"});
	
	private JButton createGraphsButton = new JButton("create subgraphs");
	
	private JButton refreshButton = new JButton("Refresh");
	
	private ApplicationInterface<V, E> application;
	
	private Set<V> selectedElements = new HashSet<V>();
	
	private Graph<V,E> currentGraph;
	
	private CentiLib<V,E> centilib;

	private JLabel currentGraphLabel = new JLabel("Graph: ");
	
	private JLabel attributesLabel = new JLabel("Edge Weights: ");
	
	private JLabel numVerticesLabel = new JLabel("Vertices: ");

    private JLabel numDirEdgesLabel = new JLabel("Directed Edges: ");

    private JLabel numUDirEdgesLabel = new JLabel("Undirected Edges: ");

    private JLabel connectedLabel = new JLabel("Connected: ");

    private JLabel loopFreeLabel = new JLabel("Loop-free: ");

    private JLabel simpleLabel = new JLabel("Simple: ");

    private JLabel avgDistanceLabel = new JLabel("Avg. Distance: ");
    
    private JLabel clusteringCoefficientLabel = new JLabel("Clustering Coefficient: ");
    
    private JLabel diameterLabel = new JLabel("Graph Diameter: ");
    
    private JLabel wienerIndexLabel = new JLabel("Wiener Index: ");
    
    private JLabel connectedCompsLabel = new JLabel("Connected Components: ");
    
    
    private JLabel currentGraphText = new JLabel("");
    
    private JLabel numVerticesText = new JLabel("");

    private JLabel numDirEdgesText = new JLabel("");

    private JLabel numUDirEdgesText = new JLabel("");

    private JCheckBox connectedCB = new JCheckBox();

    private JCheckBox loopFreeCB = new JCheckBox();

    private JCheckBox simpleCB = new JCheckBox();
    
    private JLabel avgDistanceText = new JLabel("");
    
    private JLabel clusteringCoefficientText = new JLabel("");
    
    private JLabel diameterText = new JLabel("");
    
    private JLabel wienerIndexText = new JLabel("");

	
	
	public GraphStatisticsPanel(CentiLib<V,E> centilib){
		super();
		this.centilib = centilib;
		this.application = centilib.getAppInterface();
		
		//setBorder(BorderFactory.createLineBorder(Color.BLACK, 1));
        //setPreferredSize(new Dimension(150, 500));
        setLayout(new GridBagLayout());
		
		// **** graph statistics
		JPanel textPanel = createTextPanel();
		
		attributesComboBox.addActionListener(this);
		refreshButton.addActionListener(this);
        connectedCB.setEnabled(false);
        connectedCB.setSelected(false);
        loopFreeCB.setEnabled(false);
        simpleCB.setEnabled(false);
 		
        GridBagConstraints gbcs = new GridBagConstraints();
        gbcs.fill = GridBagConstraints.HORIZONTAL;
        gbcs.weightx = 0.5;
        gbcs.weighty = 0;
        gbcs.gridy = GridBagConstraints.RELATIVE;
        gbcs.gridx = 0;
        add(textPanel,gbcs);
        
        // **** connected components
		connectedComboBox.addActionListener(this);
		createGraphsButton.addActionListener(this);
		compTable.getSelectionModel().addListSelectionListener(this);
		
        compTable.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridy = GridBagConstraints.RELATIVE;
        gbc.gridx = 0;
        
        gbc.weightx= 0;
        gbc.weighty= 0;
        gbc.fill= GridBagConstraints.HORIZONTAL;
        
        add(new JLabel("  "),gbc);
        add(connectedCompsLabel,gbc);
        add(connectedComboBox,gbc);
        
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.BOTH;
        add(new JScrollPane(compTable), gbc);
        
        gbc.weightx= 0;
        gbc.weighty= 0;
        gbc.fill= GridBagConstraints.HORIZONTAL;
        createGraphsButton.setBorder(new LineBorder(Color.GREEN, 1));
        add(createGraphsButton,gbc);
        
        validate();
        
	}
	
	public void setGraph(Graph<V,E> graph){

		this.currentGraph = graph;
		this.connectedComboBox.setSelectedIndex(0);
		setComponents(null);
		updateStats();
		refreshAttributes();
	}
	
	private void updateStats(){

        if(currentGraph == null){
        	reset();
        	return;
        }
                
        // The graph label
        currentGraphText.setText(GraphCachingUtils.getGraphName(currentGraph));
        
        // The numbers
        numVerticesText.setText(Integer.toString(currentGraph.getVertexCount()));
        numDirEdgesText.setText(Integer.toString(currentGraph.getEdgeCount(EdgeType.DIRECTED)));
        numUDirEdgesText.setText(Integer.toString(currentGraph.getEdgeCount(EdgeType.UNDIRECTED)));
        
        // The check boxes.
        // The boxes are slightly more complicated, therefore we use cache the result.
        connectedCB.setSelected(GraphCachingUtils.isConnected(currentGraph));
        loopFreeCB.setSelected(!GraphCachingUtils.containsSelfLoops(currentGraph));
        simpleCB.setSelected(!GraphCachingUtils.containsParallelEdges(currentGraph));
        
        Double value;
        // The graph centralities
        
        value = CentralityHandler.getGraphCentralityValue(currentGraph, CentralityHandler.AVERAGEDISTANCE, false); 
        if( value == null)
        	avgDistanceText.setText("not computed");
        else avgDistanceText.setText(String.valueOf(value));
        
        value = CentralityHandler.getGraphCentralityValue(currentGraph, CentralityHandler.CLUSTERINGCOEFFICIENT, false);
        if(value == null)
        	clusteringCoefficientText.setText("not computed");
        else clusteringCoefficientText.setText(String.valueOf(value));
        
        value = CentralityHandler.getGraphCentralityValue(currentGraph, CentralityHandler.DIAMETER, false);
        if(value == null)
        	diameterText.setText("not computed");
        else diameterText.setText(String.valueOf(value));
        
        value = CentralityHandler.getGraphCentralityValue(currentGraph, CentralityHandler.WIENERINDEX, false);
        if(value == null)
        	wienerIndexText.setText("not computed");
        else wienerIndexText.setText(String.valueOf(value));
        
	}

	private void reset(){
		
		// The stats
		currentGraphText.setText("");
		numVerticesText.setText("");
		numDirEdgesText.setText("");
		numUDirEdgesText.setText("");
		
		connectedCB.setSelected(false);
		loopFreeCB.setSelected(false);
		simpleCB.setSelected(false);
		
		avgDistanceText.setText("");
		clusteringCoefficientText.setText("");
		diameterText.setText("");
		wienerIndexText.setText("");
		
		// Components
		this.setComponents(null);
	}
	public void setComponents(Set<Set<V>> comps){
		TableModel t;


		if(comps == null && compTableModel == null) return;
		
    	if(comps != null && comps.size() > 0){
    		compTableModel = new ConnectedComponentTableModel<V>(comps);
    		t = compTableModel;
    	}
    	else{
    		compTableModel = null;
    		this.connectedComboBox.setSelectedIndex(0);
    		t = new DefaultTableModel();
    	}
    	
    	compTable.setModel(t);
    	compTable.tableChanged(new TableModelEvent(t));
    	validate();
	}

	@Override
	public void valueChanged(ListSelectionEvent e) {
		
		if(e.getValueIsAdjusting()) return;
		
		Set<V> selected = getSelectedVertices();
		application.selectNodes(selected);
	}
	
	
	boolean performingAction = false;
	
	@Override
	public void actionPerformed(ActionEvent ae) {
		if(performingAction)
			return;
		performingAction = true;
		
		if(ae.getSource() == attributesComboBox){
			setEdgeWeights();
			performingAction = false;
			return;
		}
			
		
		// if we are resetting
		if(connectedComboBox.getSelectedIndex() == 0){
			performingAction = false;
			return;
		}
		
		if(currentGraph == null){
			connectedComboBox.setSelectedIndex(0);
			centilib.showInformationDialog("Error", "No graph available.");
			performingAction = false;
			return;
		}
		
		// get connected components
		if(ae.getSource() == connectedComboBox){
			if(connectedComboBox.getSelectedIndex() == 1)
				setComponents(GraphCachingUtils.getConnectedComponents(currentGraph));
			if(connectedComboBox.getSelectedIndex() == 2)
				setComponents(GraphCachingUtils.getStrongConnectedComponents(currentGraph));
		}
		
		// create subgraphs
		if(ae.getSource() == createGraphsButton){
			
			Set<Set<V> > comps = getSelectedComponents();
			if(comps == null || comps.size() == 0)
				centilib.showInformationDialog("error", "Please select at least one of the components.");
			ConnectedComponents<V, E> compAction = new ConnectedComponents<V, E>(currentGraph, comps);
			
			application.createGraphsFromComps(compAction);
			
		}
		performingAction = false;
	}
	
	public void setEdgeWeights(){
		
		if(currentGraph == null || refreshingAttributes || attributesComboBox.getItemCount() == 0)
			return;
		if(attributesComboBox.getSelectedItem() instanceof DefaultEdgeAttribute){
			GraphCachingUtils.setEdgeWeights(currentGraph, null);
			centilib.refreshPanel();
			return;
		}
		
		Map<E,Number> weights = new HashMap<E,Number>();
		DoubleAttribute<E> attribute;
		
		if(attributesComboBox.getSelectedItem() instanceof DoubleAttribute<?>)
			attribute = (DoubleAttribute<E>) attributesComboBox.getSelectedItem();
		else return;
		
		for(E edge : currentGraph.getEdges()){
			Double value = attribute.getValue(edge);
			if(value == null || value.isNaN() || value.isInfinite()){
				centilib.showInformationDialog("Error", "Couldn't read the attribute " +
						"'"+attribute+"'");
				attributesComboBox.setSelectedIndex(0);
				return;
			}
			
			// edge weights must be positive
			if(attribute.getValue(edge) <0){
				centilib.showInformationDialog("Error", "The edge weights must be positive.");
				refreshAttributes();
				return;
			}
				
			weights.put(edge, attribute.getValue(edge));
		}
		GraphCachingUtils.setLastEdgeAttribute(currentGraph, attribute);
		GraphCachingUtils.setEdgeWeights(currentGraph, weights);
		centilib.refreshPanel();
	}
	
	boolean refreshingAttributes = false;
	
	public void refreshAttributes(){
		if(refreshingAttributes) return;
		refreshingAttributes = true;
		attributesComboBox.removeAllItems();
		attributesComboBox.addItem(new DefaultEdgeAttribute());
		
		if(currentGraph == null){
			refreshingAttributes = false;
			return;
		}
		
		DoubleAttribute<E> last = GraphCachingUtils.getLastEdgeAttribute(currentGraph);
		
		Set<DoubleAttribute<E> > attributes = application.getNumericEdgeAttributes(currentGraph);
		
		if(attributes == null || attributes.size() == 0){
			refreshingAttributes = false;
			return;
		}
			
		for(DoubleAttribute<E> attribute : attributes){
			attributesComboBox.addItem(attribute);
			
			if(last != null && last.toString().equals(attribute.toString()))
				attributesComboBox.setSelectedIndex(attributesComboBox.getItemCount()-1);
		}

		attributesComboBox.validate();
		refreshingAttributes = false;
	}
	
	public Set<Set<V> > getSelectedComponents(){
		
		Set<Set<V> > comps = new HashSet<Set<V> >();
		
		Set<V> tempComp;
		
        ListSelectionModel lsm = (ListSelectionModel) this.compTable.getSelectionModel();
        
        if (compTableModel == null || lsm.isSelectionEmpty())
        	return (Set<Set<V> >) Collections.unmodifiableSet(comps);

        // Find out which indexes are selected.
        int minIndex = lsm.getMinSelectionIndex();
        int maxIndex = lsm.getMaxSelectionIndex();

        for (int i = minIndex; i <= maxIndex; i++) {
            if (lsm.isSelectedIndex(i)) {
            	tempComp = compTableModel.getCompAtPos(i);
            	if(tempComp == null) continue;
            	comps.add(tempComp);
            }
        }
		
		return (Set<Set<V> >) Collections.unmodifiableSet(comps);
	}
	
    public Set<V> getSelectedVertices() {

    	Set<V> comp;
    	selectedElements.clear();
    	
        ListSelectionModel lsm = (ListSelectionModel) this.compTable.getSelectionModel();
        
        if (compTableModel == null || lsm.isSelectionEmpty())
        	return (Set<V>) Collections.unmodifiableSet(selectedElements);

        // Find out which indexes are selected.
        int minIndex = lsm.getMinSelectionIndex();
        int maxIndex = lsm.getMaxSelectionIndex();

        for (int i = minIndex; i <= maxIndex; i++) {
            if (lsm.isSelectedIndex(i)) {
            	comp = compTableModel.getCompAtPos(i);
            	if(comp == null) continue;
            	
            	for(V v : comp)
            		selectedElements.add(v);
            }
        }
        

    	return (Set<V>) Collections.unmodifiableSet(selectedElements);
    }

    private JPanel createTextPanel() {
        JPanel p = new JPanel();
        
        p.setLayout(new GridLayout(13, 2));
        
        p.add(currentGraphLabel);
        p.add(currentGraphText);
        currentGraphText.setName("CurrentGraph");
        
        p.add(attributesLabel);
        p.add(attributesComboBox);
        
        p.add(numVerticesLabel);
        p.add(numVerticesText);
        numVerticesText.setName("NumVertices");

        p.add(numDirEdgesLabel);
        p.add(numDirEdgesText);
        numDirEdgesText.setName("NumDirEdges");

        p.add(numUDirEdgesLabel);
        p.add(numUDirEdgesText);
        numUDirEdgesText.setName("NumUDirEdges");

        p.add(connectedLabel);
        p.add(connectedCB);
        connectedCB.setName("Connected");

        p.add(loopFreeLabel);
        p.add(loopFreeCB);
        loopFreeCB.setName("LoopFree");

        p.add(simpleLabel);
        p.add(simpleCB);
        simpleCB.setName("Simple");
        
        
        // graph centralities
        p.add(new JLabel(" "));
        p.add(new JLabel(" "));
        
        p.add(avgDistanceLabel);
        p.add(avgDistanceText);
        avgDistanceText.setName("AvgDistance");
        
        p.add(clusteringCoefficientLabel);
        p.add(clusteringCoefficientText);
        clusteringCoefficientText.setName("ClusteringCoefficient");
        
        p.add(diameterLabel);
        p.add(diameterText);
        diameterText.setName("Diameter");
        
        p.add(wienerIndexLabel);
        p.add(wienerIndexText);
        wienerIndexText.setText("WienerIndex");

//        p.add(rankerNameText);
//        rankerNameText.setName("RankerName");
        
        

        return p;
    }
}
