package de.ipk_gatersleben.ag_nw.centilib.gui;


/*
 * Possibly use static methods
 */

import java.awt.event.ActionEvent;

import javax.swing.Action;
import javax.swing.ButtonGroup;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.CentroidValuesCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.ClosenessCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.ClosenessVitalityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.CurrentFlowBetweennessCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.CurrentFlowClosenessCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.EccentricityCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.EigenvectorCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.HITSCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.HubbellIndexCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.InOutDegreeCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.KatzStatusIndexCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.PageRankCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.RadialityCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.SPBetweennessCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.StressCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.graphstatistics.AverageDistanceIndexAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.graphstatistics.GraphClusteringCoefficientAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.graphstatistics.GraphDiameterAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.graphstatistics.WienerIndexAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.gui.ShowCentralityDistributionAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.gui.ShowCentralityHistogramAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.io.LoadCentralityVectorAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.io.SaveCentralityVectorAction;

/**
 * Creates the menu structure for CentiLib.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 */
public class MenuBarFactory {

    private ButtonGroup layoutGroup = new ButtonGroup();

    private boolean unrestrictedCentiBiN;

    public MenuBarFactory() {
        this.unrestrictedCentiBiN = true;
    }

     public <V,E> JMenu createVertexCentralityMenu(CentiLib<V,E> centibin) {

        JMenu centralityMenu;
        JMenu undirectedCentralityMenu;
        JMenu directedCentralityMenu;

        centralityMenu = new JMenu("Vertex Centralities");
        undirectedCentralityMenu = new JMenu("Centralities for Undirected graphs");
        directedCentralityMenu = new JMenu("Centralities for Directed graphs");

        
        transformMenuItem(undirectedCentralityMenu, new InOutDegreeCentralityAction<V,E>(centibin));
        undirectedCentralityMenu.addSeparator();
        transformMenuItem(undirectedCentralityMenu, new EccentricityCentralityAction<V,E>(centibin));
        transformMenuItem(undirectedCentralityMenu, new ClosenessCentralityAction<V,E>(centibin));
        transformMenuItem(undirectedCentralityMenu, new RadialityCentralityAction<V,E>(centibin));
        transformMenuItem(undirectedCentralityMenu, new CentroidValuesCentralityAction<V,E>(centibin));
        undirectedCentralityMenu.addSeparator();
        transformMenuItem(undirectedCentralityMenu, new StressCentralityAction<V,E>(centibin));
        transformMenuItem(undirectedCentralityMenu, new SPBetweennessCentralityAction<V,E>(centibin));
        undirectedCentralityMenu.addSeparator();
        transformMenuItem(undirectedCentralityMenu, new CurrentFlowClosenessCentralityAction<V,E>(centibin));
        transformMenuItem(undirectedCentralityMenu, new CurrentFlowBetweennessCentralityAction<V,E>(centibin));
        undirectedCentralityMenu.addSeparator();
        transformMenuItem(undirectedCentralityMenu, new KatzStatusIndexCentralityAction<V,E>(centibin));
        transformMenuItem(undirectedCentralityMenu, new EigenvectorCentralityAction<V,E>(centibin));
        transformMenuItem(undirectedCentralityMenu, new HubbellIndexCentralityAction<V,E>(centibin));

        undirectedCentralityMenu.addSeparator();
        transformMenuItem(undirectedCentralityMenu, new ClosenessVitalityAction<V,E>(centibin));
        transformMenuItem(undirectedCentralityMenu, new PageRankCentralityAction<V,E>(centibin));
        transformMenuItem(undirectedCentralityMenu, new HITSCentralityAction<V,E>(centibin, true));
        transformMenuItem(undirectedCentralityMenu, new HITSCentralityAction<V,E>(centibin, false));
        //transformMenuItem(undirectedCentralityMenu, new SALSACentralityAction<V,E>(centibin, true));
        //transformMenuItem(undirectedCentralityMenu, new SALSACentralityAction<V,E>(centibin, false));

        // ****** directed
        transformMenuItem(directedCentralityMenu, new InOutDegreeCentralityAction<V,E>(centibin, true));
        transformMenuItem(directedCentralityMenu, new InOutDegreeCentralityAction<V,E>(centibin, false));
        directedCentralityMenu.addSeparator();
        transformMenuItem(directedCentralityMenu, new EccentricityCentralityAction<V,E>(centibin));
        transformMenuItem(directedCentralityMenu, new ClosenessCentralityAction<V,E>(centibin));
        transformMenuItem(directedCentralityMenu, new RadialityCentralityAction<V,E>(centibin));
        transformMenuItem(directedCentralityMenu, new CentroidValuesCentralityAction<V,E>(centibin));
        directedCentralityMenu.addSeparator();
        transformMenuItem(directedCentralityMenu, new StressCentralityAction<V,E>(centibin));
        transformMenuItem(directedCentralityMenu, new SPBetweennessCentralityAction<V,E>(centibin));
        directedCentralityMenu.addSeparator();
        transformMenuItem(directedCentralityMenu, new KatzStatusIndexCentralityAction<V,E>(centibin));
        transformMenuItem(directedCentralityMenu, new EigenvectorCentralityAction<V,E>(centibin));
        transformMenuItem(directedCentralityMenu, new HubbellIndexCentralityAction<V,E>(centibin));

        directedCentralityMenu.addSeparator();
        transformMenuItem(directedCentralityMenu, new ClosenessVitalityAction<V,E>(centibin));
        transformMenuItem(directedCentralityMenu, new PageRankCentralityAction<V,E>(centibin));
        transformMenuItem(directedCentralityMenu, new HITSCentralityAction<V,E>(centibin, true));
        transformMenuItem(directedCentralityMenu, new HITSCentralityAction<V,E>(centibin, false));
        //transformMenuItem(directedCentralityMenu, new SALSACentralityAction<V,E>(centibin, true));
        //transformMenuItem(directedCentralityMenu, new SALSACentralityAction<V,E>(centibin, false));

        transformMenuItem(centralityMenu, new LoadCentralityVectorAction<V,E>(centibin));
        transformMenuItem(centralityMenu, new SaveCentralityVectorAction<V,E>(centibin));
//        centralityMenu.addSeparator();
//        transformMenuItem(centralityMenu, new SaveSelectionVectorAction(centibin));

        
        centralityMenu.addSeparator();
        transformMenuItem(centralityMenu, new ShowCentralityDistributionAction<V,E>(centibin));
        transformMenuItem(centralityMenu, new ShowCentralityHistogramAction<V,E>(centibin));

        centralityMenu.addSeparator();
        centralityMenu.add(undirectedCentralityMenu);
        centralityMenu.add(directedCentralityMenu);
		
        centralityMenu.addMenuListener(new CentralityMenuListener(centibin, directedCentralityMenu,
                undirectedCentralityMenu));

        return centralityMenu;
    }

    public <V,E> JMenu createGraphCentralityMenu(CentiLib<V,E> centibin) {

        JMenu graphCentralityMenu = new JMenu("Graph Centralities");

        transformMenuItem(graphCentralityMenu, new GraphDiameterAction<V,E>(centibin));
        transformMenuItem(graphCentralityMenu, new WienerIndexAction<V,E>(centibin));
        transformMenuItem(graphCentralityMenu, new AverageDistanceIndexAction<V,E>(centibin));
        transformMenuItem(graphCentralityMenu, new GraphClusteringCoefficientAction<V,E>(centibin));

        return graphCentralityMenu;
    }

    private void transformMenuItem(JMenu menu, Action action) {
        
        //TODO license verfahren aus den namen entfernen
    	
    	String actionName = ((String) action.getValue(Action.NAME));
        action.putValue(Action.NAME, actionName.substring(4));
        menu.add(new JMenuItem(action));
    }

    private void transformMenuItem(JMenu menu, Action action, int keyNumber) {

        String actionName = ((String) action.getValue(Action.NAME));

        actionName = actionName.substring(4);
        JMenuItem jmi = new JMenuItem(action);
        jmi.setAccelerator(KeyStroke.getKeyStroke(keyNumber, ActionEvent.CTRL_MASK));
        menu.add(jmi);
    }
}