package de.ipk_gatersleben.ag_nw.centilib.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import org.apache.commons.collections15.Transformer;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.centralities.VertexCentrality;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.gui.ShowCentralityDistributionAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.gui.ShowCentralityHistogramAction;
import de.ipk_gatersleben.ag_nw.centilib.plugin.ApplicationInterface;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;
import edu.uci.ics.jung.graph.Graph;

/**
 * The information panel shown on the right side of the centibin window.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 */
@SuppressWarnings("serial")
public class ResultPanel<V,E> extends JPanel implements ListSelectionListener,ActionListener{

    private JTable rankerTable = new JTable();

    private RankerTableModel<V,E> rankerTableModel = null;
    
    private Set<V> selectedElements = new HashSet<V>();
    
    private ApplicationInterface<V, E> application;
    
    private JLabel currentGraphLabel = new JLabel("Graph: ");
    
    private JComboBox graphBox = new JComboBox();
 
	private JLabel centLabel = new JLabel("Centrality: ");    
    
    private JComboBox centBox = new JComboBox();
    
    private JLabel avgLabel = new JLabel("Avg-Value: ");
    
    private JTextField  avgText = new JTextField();
    
    private JButton distributionDialogButton = new JButton("Show Distribution Dialog");
    
    private JButton histogrammDialogButton = new JButton("Show Histogramm Dialog");

    private Graph<V,E> graph;
    
    private Transformer<V,String> toString;
    
    private boolean performingAction = false;
    
    private CentiLib<V,E> centilib;
    
    private DecimalFormat numberFormater = new DecimalFormat("#.#######");
    
    
    public ResultPanel(CentiLib<V,E> centilib) {
        super();
        
        this.centilib = centilib;
        this.application = centilib.getAppInterface();
        toString = centilib.getVertexLabelTransformer();
 
        centBox.addActionListener(this);
        graphBox.addActionListener(this);
        rankerTable.getSelectionModel().addListSelectionListener(this);
        distributionDialogButton.addActionListener(new ShowCentralityDistributionAction<V,E>(centilib));
        histogrammDialogButton.addActionListener(new ShowCentralityHistogramAction<V, E>(centilib));

        setLayout(new GridBagLayout());

        rankerTable.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(2,0,2,0);
        gbc.weightx = 0.0;
        gbc.weighty = 0.0;
        gbc.gridy = 0;//GridBagConstraints.RELATIVE;
        gbc.gridx = 0;
        add(currentGraphLabel,gbc);
        gbc.gridx=1;
        add(graphBox,gbc);
        
        gbc.gridy = GridBagConstraints.RELATIVE;
        gbc.gridx = 0;
        add(centLabel,gbc);
        gbc.gridx = 1;
        add(centBox,gbc);
        
        gbc.gridx = 0;
        add(avgLabel,gbc);
        gbc.gridx = 1;
        avgText.setEditable(false);
        add(avgText,gbc);


        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.gridx = 0;
        //gbc.gridy = 0;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.fill = GridBagConstraints.BOTH;
        add(new JScrollPane(rankerTable), gbc);
        
        gbc.weightx = 0;
        gbc.weighty = 0;
        //gbc.gridy = 0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        add(distributionDialogButton,gbc);
        add(histogrammDialogButton,gbc);
        
   }
    

    public void setStatus(Graph<V,E> graph) {
        if(graph == null){
        	reset();
        	return;
        }
        
        this.graph = graph;
                
		setScorer(null, "");
        updateGraphs();
        //updateCents();
    }
    
    public void setScorer(VertexCentrality<V,E> ranker, String rn) {
    	
    	if(graph == null) return; 
    	
    	performingAction = true;
    	if (ranker != null) {
            rankerTableModel = new RankerTableModel<V,E>(ranker,toString,graph);
            avgText.setText(numberFormater.format(ranker.getAverageValue()));
        } else {
        	if(GraphCachingUtils.getLastScorerName(graph).equals("")){
            	rankerTableModel = null;
            	avgText.setText("");
        	}
        	else{
        		VertexCentrality<V,E> scorer = GraphCachingUtils.getLastScorer(graph);
        		rankerTableModel = new RankerTableModel<V,E>(scorer, toString, graph);
        		avgText.setText(numberFormater.format(scorer.getAverageValue()));
        	}
        }

    	updateTable();
    	updateCents();
    	
    	performingAction = false;
        //selectedElements.clear();
    }
    
    public void setSelectedVertices(Collection<V> selected){
    	if(selected == null || selected.size() == 0 || rankerTableModel == null)
    		return;
    	
    	performingAction = true;
    	rankerTable.clearSelection();
    	selectedElements.clear();
    	for(V v : selected){
    		selectedElements.add(v);
    		int pos = rankerTableModel.getPos(v);
    		if(pos >= 0)
    			rankerTable.changeSelection(pos, 0, true, false);
    	}
    	
    	performingAction = false;
    }
    
    private void updateGraphs(){
    	
    	Collection<Graph<V,Object> > graphs = GraphCachingUtils.getGraphs();
    	
    	if(graph == null || graphs == null || graphs.size() == 0){
    		graphBox.removeAllItems();
    		graphBox.addItem("No graph available");
    		graphBox.validate();
    		return;
    	}
    	
    	if(performingAction)
    		return;
    	performingAction = true;
    	
    	graphBox.removeAllItems();
    	
    	for(Graph<V,Object> g : graphs){
    		graphBox.addItem(new StringGraph(g));//GraphCachingUtils.getGraphName(g));
    		if( g == this.graph)
    			graphBox.setSelectedIndex(graphBox.getItemCount()-1);
    	}
    	
    	graphBox.validate();
    	validate();
    	
    	performingAction = false;
    }
    
    private void updateTable(){
    	TableModel t;
    	if(rankerTableModel != null)
    		t = rankerTableModel;
    	else t = new DefaultTableModel();
    	
    	//if vertices were selected, remember them
    	Collection<V> temp = new HashSet<V>();
    	for(V v : selectedElements)
    		temp.add(v);
    	
    	rankerTable.setModel(t);
    	rankerTable.tableChanged(new TableModelEvent(t));
    	rankerTable.validate();
    	setSelectedVertices(temp);
    	validate();
    }

    private void updateCents(){
    	
    	if(graph == null){
    		centBox.removeAllItems();
    		centBox.addItem("No graph selected");
    		centBox.validate();
    		return;
    	}
    	
    	centBox.removeAllItems();
    	
    	Collection<String> cents = GraphCachingUtils.getAvailableScorerNames(graph);
    	    	
    	if(cents.size() == 0)
    		centBox.addItem("No centralities available for this graph");
    	else{
    		String last = GraphCachingUtils.getLastScorerName(graph);
    		for(String s : cents){
    			centBox.addItem(s);
    			if(s.equals(last))
    				centBox.setSelectedIndex(centBox.getItemCount()-1);
    		}
    		
    	}
    	centBox.validate();
    }
    
    public Set<V> getSelectedVertices() {

    	selectedElements.clear();

        if (rankerTableModel != null) {
            ListSelectionModel lsm = (ListSelectionModel) rankerTable.getSelectionModel();
            if (lsm.isSelectionEmpty()) {
            	return (Set<V>) selectedElements;
            }

            // Find out which indexes are selected.
            int minIndex = lsm.getMinSelectionIndex();
            int maxIndex = lsm.getMaxSelectionIndex();

            for (int i = minIndex; i <= maxIndex; i++) {
                if (lsm.isSelectedIndex(i)) {
                    selectedElements.add(rankerTableModel.getVertexAtPos(i));
                }
            }
        }

    	return (Set<V>) Collections.unmodifiableSet(selectedElements);
    }

    private void reset(){

        // The numbers
/*        numVerticesText.setText("");
        numDirEdgesText.setText("");
        numUDirEdgesText.setText("");
 
        // The check boxes
        connectedCB.setSelected(false);
        loopFreeCB.setSelected(false);
        simpleCB.setSelected(false);
*/        
        rankerTableModel = null;
        avgText.setText("");
        graph = null;
        updateTable();
        updateCents();
        updateGraphs();

    	selectedElements.clear();
    	
    	validate();
    }
    
	@Override //ListSelectionListener -> node(s) selected
	public void valueChanged(ListSelectionEvent e) {
		if(e.getValueIsAdjusting()) return;
		if(performingAction)
			return;
		
		application.selectNodes(getSelectedVertices());
		
		
		
	}

	@Override //ActionListener -> centrality selected
	public void actionPerformed(ActionEvent e) {
		
		if(performingAction)
			return;
		// **** graphBox
		if(e.getSource() == graphBox){
			Graph<V,E> selected = null;
			if(graphBox.getSelectedItem() instanceof ResultPanel<?,?>.StringGraph<?,?>) 
				selected = ((StringGraph) graphBox.getSelectedItem()).getGraph(); 
			
			if( selected != null && selected != this.graph){
				performingAction = true;
				application.selectGraph(selected);
				
				this.graph = selected;
				performingAction = false;
				return;
			}
		}
			
		// **** centBox
		if(e.getSource() == centBox){
			String cent = (String) centBox.getSelectedItem();
			if( cent == null || cent.equals("") || cent.equals(GraphCachingUtils.getLastScorerName(graph)) || GraphCachingUtils.getScorer(graph, cent) == null)
				return;
			VertexCentrality<V,E> scorer =  GraphCachingUtils.getScorer(graph, cent);
			centilib.setCurrentRanker(scorer, cent);
		}
		
	}
	
	private class StringGraph<V,E>{
		
		private Graph<V,E> graph;
		public StringGraph(Graph<V,E> graph){
			this.graph = graph;
		}
		
		@Override
		public String toString(){
			return GraphCachingUtils.getGraphName(graph);
		}
		
		public Graph<V,E> getGraph(){
			return graph;
		}
	}
	
    
}