package de.ipk_gatersleben.ag_nw.centilib.gui.actions;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.Icon;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.gui.utils.SwingWorker;
import de.ipk_gatersleben.ag_nw.centilib.utils.ErrorHandler;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;
import edu.uci.ics.jung.graph.Graph;

public abstract class AssistancePanelAction<V,E> extends CentiLibBackgroundAction<V, E> {

	private String output;
    public AssistancePanelAction(CentiLib<V,E> centibin, String name, Icon icon) {
        super (centibin, name, icon);
        output = name;
    }
    
    public AssistancePanelAction(CentiLib<V, E> centibin, String name) {
		super(centibin, name);
		output = name;
	}

    public SwingWorker run(final Collection<Graph<V,E> > graphs, final List<String> centralities){
        
    	getCentiLib().startLongRunningOperation("Action performed", CentiLib.WAIT_STRING);
    	
        SwingWorker worker = new SwingWorker() {

            private Throwable thrownException = null;

            // Running on the new thread!
            public Object construct() {
                try {
                    doItInBackground(graphs, centralities);
                } catch (Throwable exc) {
                    thrownException = exc;
                }
                return null;
            }

            // Running on the Event Thread
            public void finished() {
                super.finished();

                // This has to happen on the Swing thread!!
                if (thrownException != null) {
                    getCentiLib().holdLongRunningOperation("Exception catched", "Please confirm the dialog...");
                    ErrorHandler.getInstance().exceptionCatched(thrownException);
                    getCentiLib().continueLongRunningOperation();
                }

                getCentiLib().finishedLongRunningOperation();
            }
        };

        // Let the ball roll
        worker.start();
    	return worker;
    }
    
    public Collection<String> getPossibleCentralities(Collection<Graph<V,E> > graphs){
    	
    	Set<String> cents = new HashSet<String>();
    	
    	for(Graph<V,E> graph : graphs)
    		cents.addAll(GraphCachingUtils.getAvailableScorerNames(graph));
    	
    	return cents;
    }
    
    @Override
    public String toString(){
    	return output;
    }
    
    public abstract void doItInBackground(Collection<Graph<V,E> > graphs, List<String> cents)throws Exception;
    
}
