package de.ipk_gatersleben.ag_nw.centilib.gui.actions;

import java.awt.event.ActionEvent;

import javax.swing.Action;
import javax.swing.Icon;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.gui.utils.SwingWorker;
import de.ipk_gatersleben.ag_nw.centilib.utils.ErrorHandler;

/**
 * Asynchronously running action.
 *
 * @author Johannes Graessler
 * @author Dirk Koschuetzki
 */
@SuppressWarnings("serial")
public abstract class CentiLibBackgroundAction<V,E> extends CentiLibAction<V,E> {

    public CentiLibBackgroundAction(CentiLib<V,E> centibin, String name) {
        super(centibin, name);
    }

    public CentiLibBackgroundAction(CentiLib<V,E> centibin, String name, Icon icon) {
        super(centibin, name, icon);
    }

    public void actionPerformed(final ActionEvent actionEvent) {

        final String operationName = (String) getValue(Action.NAME);
        //checkLicense();
        
        getCentiLib().startLongRunningOperation(operationName, CentiLib.WAIT_STRING);

        SwingWorker worker = new SwingWorker() {

            private Throwable thrownException = null;

            // Running on the new thread!
            public Object construct() {
                try {
                    doItInBackground(actionEvent);
                } catch (Throwable exc) {
                    thrownException = exc;
                }
                return null;
            }

            // Running on the Event Thread
            public void finished() {
                super.finished();

                // This has to happen on the Swing thread!!
                if (thrownException != null) {
                    getCentiLib().holdLongRunningOperation("Exception catched", "Please confirm the dialog...");
                    ErrorHandler.getInstance().exceptionCatched(thrownException);
                    getCentiLib().continueLongRunningOperation();
                }

                getCentiLib().finishedLongRunningOperation();
            }
        };

        // Let the ball roll
        worker.start();
    }

    protected abstract void doItInBackground(ActionEvent e) throws Exception;

}