package de.ipk_gatersleben.ag_nw.centilib.gui.actions;

import java.awt.event.ActionEvent;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.centralities.VertexCentrality;
import de.ipk_gatersleben.ag_nw.centilib.utils.CentralityHandler;
import edu.uci.ics.jung.graph.Graph;

/**
 * Common basis for all centrality actions.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 * @author Michael Telgkamp
 */
@SuppressWarnings("serial")
public abstract class CentralityAction<V,E> extends CentiLibBackgroundAction<V,E> {

	protected boolean conditionsChecked = false;
	
	protected VertexCentrality<V,E> scorer;
	
	protected String scorerName = "VertexScorer";
	
    public CentralityAction(CentiLib<V,E> centibin, String name) {
        super(centibin, name);
    }

    public void doItInBackground(ActionEvent e) throws Exception {

		Graph<V,E> graph = getCentiLib().getCurrentGraph();
    	scorer = CentralityHandler.getScorer(graph, scorerName, null,true);
    	
        if(scorer == null)
        	conditionsChecked = false;
        
    	if(!conditionsChecked){
    		if (!isPossible())
    			return;
    		
    		if(compute(graph)){
    			conditionsChecked = true;
    		}
    	}
    	
    	if(scorer != null)
    		getCentiLib().setCurrentRanker(scorer, scorerName);
    	
    	scorer = null;
    }

    public boolean isPossible(){
    	boolean result = CentralityHandler.isPossible(getCentiLib().getCurrentGraph(), scorerName);
    	if(!result)
    		getCentiLib().showInformationDialog("Error", CentralityHandler.getErrorMessage());
    	return result;
    }

    protected boolean compute(Graph<V,E> graph) throws Exception{
	// if return true: scorer is saved in and loaded from GraphCachingUtils -> compute() is not called again
    // should assign an instance to the local member 'scorer'
    	scorer = CentralityHandler.getScorer(getCentiLib().getCurrentGraph(), scorerName, null,true);
    	if(scorer != null) return true;
    	return false;
    }
}
