package de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities;

import org.apache.commons.collections15.Transformer;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.centralities.CentroidValues;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.CentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.utils.CentralityHandler;
import edu.uci.ics.jung.algorithms.shortestpath.DijkstraDistance;
import edu.uci.ics.jung.algorithms.shortestpath.Distance;
import edu.uci.ics.jung.algorithms.shortestpath.UnweightedShortestPath;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computes the centroid values centrality.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 * @author Michael Telgkamp
 */
@SuppressWarnings("serial")
public class CentroidValuesCentralityAction<V,E> extends CentralityAction<V,E> {

	public CentroidValuesCentralityAction(CentiLib<V,E> centibin) {
        super(centibin, "*** Centroid Values");
        scorerName = CentralityHandler.CENTROID;
    }

    public static <V,E> CentroidValues<V,E> calculateCentroidValues(Graph<V,E> graph, Transformer<E,? extends Number> edgeWeights){
    	Distance<V> distance;
    	
    	if(edgeWeights == null)
    		distance = new UnweightedShortestPath<V,E>(graph);
    	else
    		distance = new DijkstraDistance<V, E>(graph,edgeWeights,true);
    	
        return calculateCentroidValues(graph, distance);
        
    }
    
    public static <V,E> CentroidValues<V,E> calculateCentroidValues(Graph<V,E> graph, Distance<V> distance){
    	CentroidValues<V, E> cv = new CentroidValues<V,E>(graph,distance);
        cv.getVertexScore(graph.getVertices().iterator().next());
    	return cv;
    }
}