package de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.centralities.ClosenessVitality;
import de.ipk_gatersleben.ag_nw.centilib.centralities.VertexCentrality;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.CentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphPreconditionChecks;
import edu.uci.ics.jung.algorithms.shortestpath.Distance;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computes the closeness vitality centrality
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 * @author Michael Telgkamp
 */
public class ClosenessVitalityAction<V,E> extends CentralityAction<V,E> {

    public ClosenessVitalityAction(CentiLib<V,E> centibin) {
        super(centibin, "*** Closeness Vitality");
        scorerName = "Closeness Vitality";
    }

    public boolean isPossible() {
        return GraphPreconditionChecks.checkPreconditionsStrongConLFSimple(getCentiLib());
    }

    public boolean compute(Graph<V,E> graph) {

        try {
            scorer = calculateClosenessVitality(graph,GraphCachingUtils.getDistance(graph));
        } catch (IllegalArgumentException e) {
            getCentiLib().showInformationDialog("Computation not possible",
                    "Closeness vitality centrality can not be computed for this Graph.\n Error message:\n"+e.getMessage());
            return false;
        }
        return true;
    }

    public static <V,E> VertexCentrality<V,E> calculateClosenessVitality(Graph<V,E> graph,Distance<V> distance) {
        ClosenessVitality<V,E> clsV = new ClosenessVitality<V,E>(graph,distance);
        clsV.getVertexScore(graph.getVertices().iterator().next());
        return clsV;
    }

}