package de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities;

import java.awt.event.ActionEvent;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.Icon;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.centralities.parameter.CentralityParameter;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.AssistancePanelAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.utils.SwingWorker;
import de.ipk_gatersleben.ag_nw.centilib.utils.CentralityHandler;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;
import edu.uci.ics.jung.graph.Graph;

public class ComputeCentralitiesAction<V, E> extends AssistancePanelAction<V, E> {

	public ComputeCentralitiesAction(CentiLib<V, E> centibin, String name,
			Icon icon) {
		super(centibin, "*** Compute centralities", icon);

	}

	public ComputeCentralitiesAction(CentiLib<V, E> centibin, String name) {
		super(centibin, name);
		
	}

	@Override
	public void doItInBackground(Collection<Graph<V, E>> graphs,
			List<String> cents) throws Exception {

		if(graphs == null || cents == null)
			return;
		
		String errorMessage = "";
		String errors;
		Map<Graph<V,E>, Map<String, CentralityParameter > > params = new HashMap<Graph<V,E>, Map<String,CentralityParameter > >();
		
		Set<String> deletedCents = new HashSet<String>();
			
		// **** check which centralities can be computed for the graphs
		for(Graph<V,E> g : graphs){
			if(g.getVertexCount() == 0){
				errorMessage += "\n\nGraph: "+GraphCachingUtils.getGraphName(g)+"\n   no vertices found!\n";
				continue;
			}
			errors = "\n";
			params.put(g, new HashMap<String,CentralityParameter >());
			
			for(String cent : cents){
				getCentiLib().startLongRunningOperation("", "checking preconditions: "+cent);
				if(!CentralityHandler.isPossible(g, cent)){
					errors+= "\n   "+cent+"\n"
						+CentralityHandler.getErrorMessage()+"\n";
				}
				else{
					CentralityParameter param = CentralityHandler.getParameter(getCentiLib(),g, cent, true);
					if(param != null && (param.getValue()==null || param.getValue().isNaN())){
						getCentiLib().showInformationDialog("Aborted", "No valid value entered, '"+cent+"' will be skipped!");
						deletedCents.add(cent);
					}
					params.get(g).put(cent, param);
				}
			}
			if(!errors.equals("\n"))
				errorMessage += "\n\nGraph: "+GraphCachingUtils.getGraphName(g)+errors;
		}
		
		getCentiLib().finishedLongRunningOperation();
		
		for(String s : deletedCents)
			cents.remove(s);
		
		if(!errorMessage.equals(""))
			getCentiLib().showInformationDialog("errors", "The following errors occured.\nImpossible centralities" +
					" will be skipped!"+errorMessage);
		
		int sum = 0;
		
		for(Graph<V,E> graph : graphs)
			sum += graph.getVertexCount();
		
		// if there are many nodes (>700) and more then 2
		// centralities to be computed --> ask for starting
		if(sum > 700 && cents.size()>2)
			if(!getCentiLib().askForBoolean("About to start the computation. This could take a while.\n\nAre you sure?")){

				getCentiLib().showInformationDialog("aborted", "Action aborted by user. No centralities were computed!");
				return;
			}
		
		Set<SwingWorker> workers = new HashSet<SwingWorker>();
		
		// **** compute the possible centralities for the graphs
		getCentiLib().startLongRunningOperation("", "calculating centralities");
		CentralityHandler.useBrandes(true);
		
		for(final Graph<V,E> g : graphs){
			if(g.getVertexCount() == 0)
				continue;
			for(final String cent : cents){
				
				final CentralityParameter param = params.get(g).get(cent);
				
				SwingWorker worker = new SwingWorker(){
					
					@Override
					public Object construct(){
						if(CentralityHandler.isPossible(g,cent))
								CentralityHandler.getScorer(g, cent, param,true);
						return null;
					}
					
					@Override
					public void finished(){
						super.finished();
						
						
					}
				};
				worker.start();
				workers.add(worker);
			}
		}
		
		for(SwingWorker worke : workers)
			worke.get();
		CentralityHandler.startBrandes();
		
		getCentiLib().finishedLongRunningOperation();
		getCentiLib().refreshPanel();
		getCentiLib().showInformationDialog("finished", "Finished ");//calculating the selected centralities\n for the selected Graphs.\n" +
//				"The results are available in the 'Result'-panel.");

	}
	

	@Override
	protected void doItInBackground(ActionEvent e) throws Exception {
		// TODO Auto-generated method stub

	}

	@Override
    public Collection<String> getPossibleCentralities(Collection<Graph<V,E> > graphs){
		
		return CentralityHandler.getCentList();
	}
}
