package de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities;

import java.util.Map;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.centralities.Eigenvector;
import de.ipk_gatersleben.ag_nw.centilib.centralities.VertexCentrality;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.IterativeCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.utils.CentralityHandler;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computes the eigenvector centrality.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 */
public class EigenvectorCentralityAction<V,E> extends IterativeCentralityAction<V,E> {

    public EigenvectorCentralityAction(CentiLib<V,E> centibin) {
        super(centibin, "*** Eigenvector");
        scorerName = CentralityHandler.EIGENVECTOR;
    }

    public boolean compute(Graph<V,E> graph) {
    	//scorer = CentralityHandler.getScorer(graph, getCentiLib().getDistance(), scorerName, null);
        scorer = calculateEigenvector(graph);
        return true;
    }

    public Eigenvector<V,E> calculateEigenvector(Graph<V,E> graph) {
     
        return new Eigenvector<V,E>(graph);
    }
    
    public static <V,E> VertexCentrality<V,E> calculateEigenvector(Graph<V,E> graph, Map<E,Number> edgeWeights){
    	
    	Eigenvector<V,E> ev = null;
    	if(edgeWeights == null)
    		ev = new Eigenvector<V, E>(graph);
    	else
    		ev = new Eigenvector<V,E>(graph,edgeWeights);
    	
    	ev.getVertexScore(graph.getVertices().iterator().next());
    	return ev;
    }
}