package de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.centralities.CentiLibHITS;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.IterativeCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.utils.CentralityHandler;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computes both of the HITS centralities.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 * @author Michael Telgkamp
 */
@SuppressWarnings("serial")
public class HITSCentralityAction<V,E> extends IterativeCentralityAction<V,E> {

    private boolean useAuthority;

    public HITSCentralityAction(CentiLib<V,E> centibin, boolean useAuthority) {
        super(centibin, "*** HITS - " + (useAuthority ? "Authority" : "Hubs"));

        this.useAuthority = useAuthority;
        scorerName = useAuthority ? CentralityHandler.HITSAUTHS : CentralityHandler.HITSHUBS;

    }

    @Override
    public boolean compute(Graph<V,E> graph) {
    	scorer = calculateHITS(graph,useAuthority);
        return true;
    }

    public CentiLibHITS<V,E> calculateHITS(Graph<V,E> graph, boolean useAuthority) {
    	CentiLibHITS<V,E> hi = (CentiLibHITS<V,E>) CentralityHandler.getScorer(getCentiLib().getCurrentGraph(),scorerName,null,true);
    	//new CentiLibHITS<V,E>(graph, useAuthority);
    	if(hi == null)
    		return null;
    	int iterations = 50, more = 0;
    	do{
        	if(hi.getIterations() >= hi.getMaxIterations())
        		more = getNumberOfIterations();
        	iterations += more;
        	calculateHITS(hi,iterations);
    	}while (more > 0);

        return hi;
    }
    
    public static <V,E> CentiLibHITS<V,E> calculateHITS(Graph<V,E> graph, boolean useAuthority, int iterations){
        CentiLibHITS<V,E> hi = new CentiLibHITS<V,E>(graph, GraphCachingUtils.getEdgeWeightTransformer(graph),useAuthority);
        calculateHITS(hi,iterations);
        return hi;
    }
    
    public static <V,E> void calculateHITS(CentiLibHITS<V,E> hits, int iterations){
    	hits.setMaxIterations(iterations);
        hits.setTolerance(IterativeCentralityAction.TOLERANCE);
        hits.evaluate();
    }
}