package de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.centralities.HubbellIndex;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.CentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.utils.CentralityHandler;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computes the Hubbell index.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 * @author Michael Telgkamp
 */

@SuppressWarnings("serial")
public class HubbellIndexCentralityAction<V,E> extends CentralityAction<V,E> {

    public HubbellIndexCentralityAction(CentiLib<V,E> centibin) {
        super(centibin, "*** Hubbell Index");
        scorerName = CentralityHandler.HUBBEL;
    }

    public boolean compute(Graph<V,E> graph) {

        Double weightFactor = getCentiLib().askForDouble("Please enter a weight factor.", "0.5", 0.0,
                Double.POSITIVE_INFINITY);
        if (weightFactor== null){
        	scorer = null;
        	return false;
        }
        
        scorerName = "Hubbell Index"+weightFactor;
        scorer = GraphCachingUtils.getScorer(graph, scorerName);
        if(scorer == null){
        	if (HubbellIndex.isSolvable(graph, GraphCachingUtils.getEdgeWeights(graph),weightFactor)) {
                scorer = calculateHubbellIndex(graph, weightFactor,true);
            } else {
                getCentiLib().showInformationDialog("Computation not possible",
                        "Hubbell Index can not be computed for this Graph.");
                return true;
            }
        }

        // always return false, so this method is called next time and the user can give a new weightFactor
        return false;
    }

    public static <V,E> HubbellIndex<V,E> calculateHubbellIndex(Graph<V,E> graph, double weightFactor) {
        HubbellIndex<V,E> hi = new HubbellIndex<V,E>(graph, GraphCachingUtils.getEdgeWeights(graph), weightFactor);
        hi.getVertexScore(graph.getVertices().iterator().next());
        return hi;
    }

    public static <V,E> HubbellIndex<V,E> calculateHubbellIndex(Graph<V,E> graph, double weightFactor,boolean isSolvable) {
        HubbellIndex<V,E> hi = new HubbellIndex<V,E>(graph, GraphCachingUtils.getEdgeWeights(graph),weightFactor,isSolvable);
        hi.getVertexScore(graph.getVertices().iterator().next());
        return hi;
    }

}
