package de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities;

import java.util.Map;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.centralities.KatzStatusIndex;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.CentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphPreconditionChecks;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computes Katz status index.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 * @author Michael Telgkamp
 */
public class KatzStatusIndexCentralityAction<V,E> extends CentralityAction<V,E> {

    public KatzStatusIndexCentralityAction(CentiLib<V,E> centibin) {
        super(centibin, "*** Katz Status Index");
        scorerName = "Katze Status Index";
    }

    public boolean isPossible() {
        return GraphPreconditionChecks.checkPreconditionsLFSimple(getCentiLib());
    }

    public boolean compute(Graph<V,E> graph) {

        double maxAlpha = KatzStatusIndex.getMaxAlphaValue(graph);

        Double alpha = getCentiLib().askForDouble("Please enter an alpha value!", Double.toString(maxAlpha / 2),
                0, maxAlpha);

        if (alpha == null){
        	scorer = null;
        	return false;
        }
        
        scorerName = "Katze Status Index"+alpha;
        scorer = GraphCachingUtils.getScorer(graph, scorerName);
        if(scorer == null)
            scorer = calculateKatzStatusIndex(graph, alpha,GraphCachingUtils.getEdgeWeights(graph));
        return false;
    }

    public static <V,E> KatzStatusIndex<V,E> calculateKatzStatusIndex(Graph<V,E> graph, double alpha,Map<E,Number> edgeWeights) {
    	KatzStatusIndex<V,E> ksi;
    	
    	if(edgeWeights == null)
    		ksi = new KatzStatusIndex<V,E>(graph,alpha);
    	else
    		ksi = new KatzStatusIndex<V,E>(graph,edgeWeights,alpha);
    	
        ksi.getVertexScore(graph.getVertices().iterator().next());
        return ksi;
    }
}