package de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities;

import org.apache.commons.collections15.Transformer;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.centralities.BrandesSP;
import de.ipk_gatersleben.ag_nw.centilib.centralities.SPBetweenness;
import de.ipk_gatersleben.ag_nw.centilib.centralities.VertexCentrality;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.CentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.utils.CentralityHandler;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computes the shortest path betweenness centrality
 *
 * @author Johannes Graessler 
 * @author Michael Telgkamp
 * @author Dirk Koschuetzki
 */
@SuppressWarnings("serial")
public class SPBetweennessCentralityAction<V,E> extends CentralityAction<V,E> {

	public SPBetweennessCentralityAction(CentiLib<V,E> centibin) {
		super(centibin, "*** Shortest-path Betweenness");
		scorerName = CentralityHandler.SPBETWEENNESS;
	}

	public static <V,E> VertexCentrality<V,E> calculateSPBetweenness(Graph<V,E> graph, Transformer<E,? extends Number> edgeWeights) {
		SPBetweenness<V, E> spb = new SPBetweenness<V, E>(graph);
		BrandesSP<V,E> brandes;
		
		if(edgeWeights == null)
			brandes = new BrandesSP<V,E>(graph);
		else
			brandes = new BrandesSP<V, E>(graph,edgeWeights);
		
		brandes.addListener(spb);
		brandes.start();
		
		//BetweennessCentrality<V, E> betw = new BetweennessCentrality<V, E>(graph);
		return spb;
	}

}