package de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities;

import org.apache.commons.collections15.Transformer;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.centralities.BrandesSP;
import de.ipk_gatersleben.ag_nw.centilib.centralities.BrandesStress;
import de.ipk_gatersleben.ag_nw.centilib.centralities.VertexCentrality;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.CentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.utils.CentralityHandler;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computes the Stress centrality
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 * @author Michael Telgkamp
 */
public class StressCentralityAction<V,E> extends CentralityAction<V,E> {

	public StressCentralityAction(CentiLib<V,E> centibin) {
		super(centibin, "*** Stress");
		scorerName = CentralityHandler.STRESS;
	}

	public boolean isPossible() {
		return CentralityHandler.isPossible(getCentiLib().getCurrentGraph(), scorerName);
        //return GraphPreconditionChecks.checkPreconditionsStrongConLFSimple(getCentiLib());
	}

	public static <V,E> VertexCentrality<V,E> calculateStress(Graph<V,E> graph,Transformer<E, ? extends Number> edgeWeights) {
		BrandesStress<V,E> stress = new BrandesStress<V,E>(graph);//new Stress<V,E>(graph,distance);
		BrandesSP<V,E> brandes;
		
		if(edgeWeights != null)
			brandes = new BrandesSP<V,E>(graph,edgeWeights);
		else
			brandes = new BrandesSP<V, E>(graph);
		
		brandes.addListener(stress);
		brandes.start();
		return stress;//new Stress<V,E>(graph,GraphCachingUtils.getDistance(graph));
	}
}
