package de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.experimental;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.centralities.SALSA;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.CentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphPreconditionChecks;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computes both of the SALSA centralities.
 * 
 * @author Dirk Koschuetzki
 * @author Michael Telgkamp
 */
public class SALSACentralityAction<V,E> extends CentralityAction<V,E> {

	boolean useAuthority;

	public SALSACentralityAction(CentiLib<V,E> centibin, boolean useAuthority) {
		super(centibin, "--- SALSA - " + (useAuthority ? "Authority" : "Hubs"));
		this.useAuthority = useAuthority;
		scorerName = "SALSA" + (useAuthority ? " - Authority" : " - Hubs");
	}

	public boolean isPossible() {
		return GraphPreconditionChecks.checkPreconditionsConLFSimple(getCentiLib());
	}

	public boolean compute(Graph<V,E> graph) {
		scorer = calculateSALSA(graph, useAuthority);
		return true;
	}

	public static <V,E> SALSA<V,E> calculateSALSA(Graph<V,E> graph, boolean useAuthority) {
		SALSA<V,E> sa = new SALSA<V,E>(graph, useAuthority);
		sa.getVertexScore(graph.getVertices().iterator().next());
		return sa;
	}
}
