package de.ipk_gatersleben.ag_nw.centilib.gui.actions.graphstatistics;

import java.text.DecimalFormat;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.graphstatistics.WienerIndex;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.CentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphPreconditionChecks;
import edu.uci.ics.jung.algorithms.shortestpath.Distance;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computes the average distance.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 * @author Michael Telgkamp
 */
public class AverageDistanceIndexAction<V,E> extends CentralityAction<V,E> {

    public AverageDistanceIndexAction(CentiLib<V,E> centibin) {
        super(centibin, "*** Average Distance");
    }

    public boolean isPossible() {
        boolean result = GraphPreconditionChecks.checkPreconditionsStrongConLFSimple(getCentiLib());
        if(!result) getCentiLib().showInformationDialog("error", GraphPreconditionChecks.getErrorMessage());
        return result;
    }

    public boolean compute(Graph<V,E> graph) {
        double avDist = calculateAverageDistance(graph,getCentiLib().getDistance());
        DecimalFormat nf = new DecimalFormat("#0.0000");

        getCentiLib().showInformationDialog("Average Distance",
                "The average distance of this graph is " + nf.format(avDist));
        return false;
    }

    public static <V,E> double calculateAverageDistance(Graph<V,E> graph, Distance<V> distance) {
        WienerIndex<V,E> wi = new WienerIndex<V,E>(graph,distance);
        int n = graph.getVertexCount();
        return (wi.getWienerIndex() / (n * (n - 1)));
    }
}
