package de.ipk_gatersleben.ag_nw.centilib.gui.actions.graphstatistics;

import java.text.DecimalFormat;
import java.util.Map;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.CentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphPreconditionChecks;
import edu.uci.ics.jung.algorithms.metrics.Metrics;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computes the clustering coefficient.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 * @author Michael Telgkamp
 */
// TODO: Write a test for the clustering coefficient
public class GraphClusteringCoefficientAction<V,E> extends CentralityAction<V,E> {

    public GraphClusteringCoefficientAction(CentiLib<V,E> centibin) {
        super(centibin, "--- Average Clustering Coefficient");
    }

    public boolean isPossible() {
        boolean result = GraphPreconditionChecks.checkPreconditionsConUndirLFSimple(getCentiLib());
        if(!result) getCentiLib().showInformationDialog("error", GraphPreconditionChecks.getErrorMessage());
        return result;
    }

    public boolean compute(Graph<V,E> graph) {

        double dia = calculateClusteringCoefficient(graph);
        DecimalFormat nf = new DecimalFormat("#0.0000");

        getCentiLib().showInformationDialog("Average Clustering Coefficient",
                "The average clustering coefficient of this graph is " + nf.format(dia));
        return false;
    }

    public static <V,E> double calculateClusteringCoefficient(Graph<V,E> graph) {
        double cc = 0;
        Map<V,Double> ccs = Metrics.clusteringCoefficients(graph);
        for(V v : graph.getVertices()) {
            cc += ccs.get(v);
        }
        return cc / ccs.size();
    }
}