package de.ipk_gatersleben.ag_nw.centilib.gui.actions.graphstatistics;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.CentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.utils.CentralityHandler;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphPreconditionChecks;
import edu.uci.ics.jung.algorithms.shortestpath.Distance;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computes the Diameter.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 * @author Michael Telgkamp
 */
public class GraphDiameterAction<V,E> extends CentralityAction<V,E> {

    public GraphDiameterAction(CentiLib<V,E> centibin) {
        super(centibin, "*** Graph Diameter");
    }

    public boolean isPossible() {
        boolean result = GraphPreconditionChecks.checkPreconditionsStrongConLFSimple(getCentiLib());
        if(!result) getCentiLib().showInformationDialog("error", GraphPreconditionChecks.getErrorMessage());
        return result;
    }

    public boolean compute(Graph<V,E> graph) {

        double dia = calculateGraphDiameter(graph, getCentiLib().getDistance());

        getCentiLib().showInformationDialog("Diameter", "The diameter of this graph is " + dia);
        return false;
    }
    
    public static <V,E> double calculateGraphDiameter(Graph<V,E> graph, Distance<V> distance){
    	return CentralityHandler.getGraphCentralityValue(graph, CentralityHandler.DIAMETER,true);
    }
}