package de.ipk_gatersleben.ag_nw.centilib.gui.actions.graphstatistics;

import java.text.DecimalFormat;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.graphstatistics.WienerIndex;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.CentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphPreconditionChecks;
import edu.uci.ics.jung.algorithms.shortestpath.Distance;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computes the Wiener Index.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 * @author Michael Telgkamp
 */
public class WienerIndexAction<V,E> extends CentralityAction<V,E> {

    public WienerIndexAction(CentiLib<V,E> centibin) {
        super(centibin, "*** Wiener Index");
    }

    public boolean isPossible() {
        boolean result = GraphPreconditionChecks.checkPreconditionsStrongConLFSimple(getCentiLib());
        if(!result) getCentiLib().showInformationDialog("error", GraphPreconditionChecks.getErrorMessage());
        return result;
    }

    public boolean compute(Graph<V,E> graph) {
        double cls = calculateWienerIndex(graph,getCentiLib().getDistance());
        DecimalFormat nf = new DecimalFormat("#0");
        String wienerIndex = nf.format(cls);

        getCentiLib().showInformationDialog("Wiener Index", "The Wiener index of this graph is " + wienerIndex);
        return false;
    }

    public static <V,E> double calculateWienerIndex(Graph<V,E> graph, Distance<V> distance) {
        return new WienerIndex<V,E>(graph,distance).getWienerIndex();
    }
}