package de.ipk_gatersleben.ag_nw.centilib.gui.actions.io;

import java.awt.event.ActionEvent;
import java.io.IOException;
import java.util.Collection;
import java.util.List;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.IOAction;
import de.ipk_gatersleben.ag_nw.centilib.io.PajekVectorFiler;
import de.ipk_gatersleben.ag_nw.centilib.io.TSVFiler;
import de.ipk_gatersleben.ag_nw.centilib.utils.CentiLibFileWrapper;
import de.ipk_gatersleben.ag_nw.centilib.utils.CentralityHandler;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;
import edu.uci.ics.jung.algorithms.scoring.VertexScorer;
import edu.uci.ics.jung.graph.Graph;

/**
 * Saves the current ranking to a file.
 * 
 * @author Johannes Graessler
 * @author Dirk Koschuetzki
 */
public class SaveCentralityVectorAction<V,E> extends IOAction<V,E> {

    public SaveCentralityVectorAction(CentiLib<V,E> centilib) {
    	super(centilib,"  to single vector files");
    }

    public SaveCentralityVectorAction(CentiLib<V,E> centilib,String name){
    	super(centilib,name);
    }
    
    public void doItInBackground(ActionEvent e) throws IOException {
    	
        Graph<V,E> graph = getCentiLib().getCurrentGraph();
        
        VertexScorer<V,Double> ranker = getCentiLib().getCurrentRanker();
        
        if (graph == null || graph.getVertexCount() == 0 || ranker == null) {
        	getCentiLib().showInformationDialog("Error", "No centrality values found!");
            return;
        }

        String recomName = (getCentiLib().getCurrentGraphName() + "_" + getCentiLib().getCurrentRanking()).replace(':',
                '-');
        CentiLibFileWrapper cfw = getCentiLib().showSaveVectorDialog(recomName);

        if (cfw != null) {

            if (cfw.isPajekVectorFile()) {
                PajekVectorFiler<V,E> filer = new PajekVectorFiler<V,E>();
                filer.save(cfw.getSelectedFile().getAbsolutePath(), ranker, graph.getVertices());
            }
            if (cfw.isTSVVectorFile()) {
                TSVFiler<V,E> filer = new TSVFiler<V,E>();
                filer.saveVector(cfw.getSelectedFile().getAbsolutePath(), ranker, graph,getCentiLib().getVertexLabelTransformer());
            }
        }

    }
    
    public void doItInBackground(Collection<Graph<V,E> > graphs, List<String> cents) throws IOException{

    	if(graphs == null || cents == null)
    		return;
    	
    	boolean result = true;
    	
    	for(Graph<V,E> graph : graphs){
			for(String cent : cents){
				
				VertexScorer<V,Double> scorer = CentralityHandler.getScorer(graph, cent, null, false);
				
				String recomName = GraphCachingUtils.getGraphName(graph).replace('*', ' ') + "-" + cent;
		
				try{
			        CentiLibFileWrapper cfw = getCentiLib().showSaveVectorDialog(recomName);
			
			        if (cfw != null) {
			
			            if (cfw.isPajekVectorFile()) {
			                PajekVectorFiler<V,E> filer = new PajekVectorFiler<V,E>();
			                filer.save(cfw.getSelectedFile().getAbsolutePath(), scorer, graph.getVertices());
			            }
			            if (cfw.isTSVVectorFile()) {
			                TSVFiler<V,E> filer = new TSVFiler<V,E>();
			                filer.saveVector(cfw.getSelectedFile().getAbsolutePath(), scorer, graph,getCentiLib().getVertexLabelTransformer());
			            }
			        }
			        else
			        	result = false;
		        }
				catch(Exception ex){
						getCentiLib().showInformationDialog("Error", "Couldn't save the file! Errors:\n\n"+ex.getMessage());
						result = false;			
				}
			}
		}
		String message;
		if(result)
			message = "Successfully exported all files.";
		else message = "Finished, but some files were skipped (due to an error or by the user).";
		
		getCentiLib().showInformationDialog("Finished", message);
    	
    }
}