package de.ipk_gatersleben.ag_nw.centilib.gui.actions.io;

import java.awt.event.ActionEvent;
import java.io.IOException;
import java.util.Collection;
import java.util.List;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.IOAction;
import de.ipk_gatersleben.ag_nw.centilib.io.PajekVectorFiler;
import de.ipk_gatersleben.ag_nw.centilib.utils.CentiLibFileWrapper;
import edu.uci.ics.jung.algorithms.scoring.VertexScorer;
import edu.uci.ics.jung.graph.Graph;

/**
 * Saves a vector of selected vertices.
 * 
 * @author Johannes Graessler
 * @author Dirk Koschuetzki
 */
public class SaveSelectionVectorAction<V,E> extends IOAction<V,E> {

    public SaveSelectionVectorAction(CentiLib<V,E> centibin) {
        super(centibin, "*** Save Selection Vector");
    }

    public void doItInBackground(ActionEvent e) throws IOException {
        Graph<V,E> graph = getCentiLib().getCurrentGraph();
        if (graph == null) {
            return;
        }

        VertexScorer<V,Double> ranker = getCentiLib().getCurrentRanker();
        if (ranker == null) {
            return;
        }

        String recomName = (getCentiLib().getCurrentGraphName() + "_Selection").replace(':',
                '-')
                + ".vec";

        CentiLibFileWrapper cfw = getCentiLib().showSaveSelectionDialog(recomName);

        if (cfw != null) {
            if (cfw.isPajekVectorFile()) {
                PajekVectorFiler<V,E> filer = new PajekVectorFiler<V,E>();
                filer.save(cfw.getSelectedFile().getAbsolutePath(),
                		getCentiLib().getCurrentRanker(),
                		getCentiLib().getSelectedVertices());
            }
        }
    }

	@Override
	public void doItInBackground(Collection<Graph<V, E>> graphs,
			List<String> cents) throws Exception {
		// TODO Auto-generated method stub
		
	}
}