package de.ipk_gatersleben.ag_nw.centilib.io;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Collection;
import java.util.LinkedList;

import org.apache.commons.collections15.BidiMap;

import de.ipk_gatersleben.ag_nw.centilib.centralities.FileBasedRanker;
import edu.uci.ics.jung.algorithms.scoring.VertexScorer;
import edu.uci.ics.jung.algorithms.util.Indexer;
import edu.uci.ics.jung.graph.Graph;

/**
 * Load and save centralities in the pajek format.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 */
public class PajekVectorFiler<V,E> {

	private static String WINDOWS_LINE_SEPARATOR = "\r\n";

	public FileBasedRanker<V,E> load(Graph<V,E> graph, String absolutePath)
			throws IOException {
		
		BufferedReader br = new BufferedReader(new FileReader(absolutePath));
		LinkedList<Double> resultList = new LinkedList<Double>();
		try {

			String line = br.readLine();
			while (line != null) {
                if (line.toLowerCase().trim().startsWith("*vertices")) {
					line = br.readLine();
					continue;
				}
				resultList.add(new Double(line.trim()));
				line = br.readLine();
			}
		} finally {
			br.close();
		}
		String centName = "";
		//use the filename as the name of the ranker
		File file = new File(absolutePath);
		FileBasedRanker<V,E> fbr = new FileBasedRanker<V,E>(file.getName(),graph, resultList);
		return fbr;
	}

	public void save(String absolutePath, VertexScorer<V,Double> ranker, Collection<V> vertices)
			throws IOException {

		//if extension isn't .vec, add it
		//if(!absolutePath.substring(absolutePath.length()-5).equals(".vec"))
			//absolutePath += ".vec";
		
		PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(
				absolutePath)));
		try {

			pw.println("*Vertices " + vertices.size());
//			pw.print("*Vertices " + graph.getVertexCount());
			// Tricky: Pajek needs the windows separator!
//			pw.print(WINDOWS_LINE_SEPARATOR);

			BidiMap<V,Integer> indexer = Indexer.create(vertices);

			for (int i = 0; i < vertices.size(); i++) {
				V vertex = indexer.getKey(i);

				// TODO: Format RankScore, perhaps according to a locale?
				pw.println(ranker.getVertexScore(vertex));
//				pw.print(ranker.getVertexScore(vertex));
				// Tricky: Pajek needs the windows separator!
//				pw.print(WINDOWS_LINE_SEPARATOR);
			}

		} finally {
			pw.close();
		}
	}
}