package de.ipk_gatersleben.ag_nw.centilib.plugin;

/**
 * The DoubleAttribute enables access to a specific attribute of any type of objects.
 * CentiLib uses this Interface to handle node and edge attributes.
 * 
 * @author Johannes Graessler
 *
 * @param <T> Type of object this Interface is an Attribute for (e.g. the type of the node data structure)
 */
public interface DoubleAttribute<T> {

	/**
	 * The method toString() should be overwritten to return the name of the attribute.
	 * For example, toString is used by the drop-down list, if the user wants to select
	 * an edge attribute as edge weights in the Graph Statistics panel.
	 * 
	 * @return the name of the attribute
	 */
	@Override public String toString();
	
	/**
	 * The name of the attribute.
	 * 
	 * @return the name of the attribute
	 */
	public String getName();
	
	/**
	 * Returns the attribute value for element if it exists, null or Double.NaN otherwise.
	 * 
	 * @param element the object to get the attribute value for
	 * 
	 * @return the attribute value for the given element
	 */
	public Double getValue(T element);
}
