package de.ipk_gatersleben.ag_nw.centilib.utils;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ConcurrentModificationException;
import java.util.Date;

import javax.swing.JFrame;
import javax.swing.SwingUtilities;

/**
 * The error handler used everywhere.
 * 
 * @author Dirk Koschuetzki
 */
public class ErrorHandler {

    private static String VERSION_INFO = "de/ipk_gatersleben/ag_nw/centibin/utils/system.txt";

    private ErrorHandlerDialog ehd;

    private static ErrorHandler instance;

    private ErrorHandler(JFrame parent) {
        ehd = new ErrorHandlerDialog(parent);
    }

    public static ErrorHandler getInstance() {
        assert (instance != null);
        return instance;
    }

    public static void init(JFrame parent) {
        instance = new ErrorHandler(parent);
    }

    public void exceptionCatched(final Throwable t) {
    	
        if (ifKnownException(t)) {
            logException(t, true);
            return;
        }

        logException(t, false);

        System.err.println(t.getMessage());
        t.printStackTrace(System.err);

    	if(true)
    		//Fehlermeldung erstmal nicht ausgeben
    		return;

        if (SwingUtilities.isEventDispatchThread()) {
            ehd.showException(t);
        } else {
            try {
                SwingUtilities.invokeAndWait(new Runnable() {
                    public void run() {
                        ehd.showException(t);
                    }
                });
            } catch (InterruptedException e1) {
                e1.printStackTrace();

                // I just quit on that. No idea what to do.
                assert (false);
            } catch (InvocationTargetException e1) {
                // there was an exception during opening the display. I do not
                // know what to do here.
                e1.printStackTrace();
                e1.getCause().printStackTrace();
                assert (false);
            }
        }
    }

    private void logException(Throwable t, boolean known) {

        File f;
        try {
            f = File.createTempFile("CentiBiN_Exception_", ".log");
            PrintStream ps = new PrintStream(new FileOutputStream(f));

            ps.println("----------------------------------------------------------------------");
            ps.println("  This is an error log from CentiLib.");
            ps.println("  You can either send this file to the koschuet@ipk-gatersleben.de ");
            ps.println("  or delete it.");
            ps.println("----------------------------------------------------------------------");
            ps.println();
            ps.println(new Date().toString());
            //ps.println(versionInformation());
            ps.println();
            ps.println("----------------------------------------------------------------------");
            if (known) {
                ps.println(" Known exception!");
                ps.println("----------------------------------------------------------------------");
            }
            ps.println(t.getMessage());
            t.printStackTrace(ps);
            ps.print("----------------------------------------------------------------------");
            ps.close();
        } catch (IOException e) {
            System.err.println("*** Fatal error: error handler could not write to a temp file!");
            System.err.println("*** Fatal error: error handler could not write to a temp file!");
            System.err.println("*** Fatal error: error handler could not write to a temp file!");
            System.err.println(t.getMessage());
            e.printStackTrace(System.err);
            System.err.println("*** Fatal error: error handler could not write to a temp file!");
            System.err.println("*** Fatal error: error handler could not write to a temp file!");
            System.err.println("*** Fatal error: error handler could not write to a temp file!");
        }
    }

    private String insertJavaVersion(String message) {
        String javaVersion = System.getProperty("java.version");
        String javaVendor = System.getProperty("java.vendor");
        return message.replaceAll("JAVA_VERSION", javaVersion).replaceAll("JAVA_VENDOR", javaVendor);
    }

    private String versionInformation() {

        URL url = this.getClass().getClassLoader().getResource(VERSION_INFO);
        URLConnection uc = null;
        BufferedReader br = null;
        
        try {
            try {
                uc = url.openConnection();
                InputStream is = (InputStream) uc.getContent();
                InputStreamReader isr = new InputStreamReader(is);
                br = new BufferedReader(isr);

                StringBuffer sb = new StringBuffer();
                String line = br.readLine();
                while (line != null) {
                    sb.append(line);
                    sb.append("\n");
                    line = br.readLine();
                }
                sb.append("\n");
                return insertJavaVersion (sb.toString());
            } finally {
                br.close();
            }

        } catch (IOException e) {
            e.printStackTrace();
            return "Resource file not found.\nError message: " + e.getMessage();
        } catch (NullPointerException e) {
            e.printStackTrace();
            return "Resource file not found.\nError message: " + e.getMessage();
        }
    }

    private boolean ifKnownException(Throwable t) {

        boolean known = false;

        // java.util.ConcurrentModificationException
        if (t instanceof ConcurrentModificationException) {
            known = false;
        }

        if (known) {
            System.err.println("Exception eliminated!");
            System.err.println(t.getMessage());
            t.printStackTrace(System.err);
        }
        return known;
    }
}