package de.ipk_gatersleben.ag_nw.centilib.utils;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import edu.uci.ics.jung.graph.Graph;

/**
 * Provides the precondition checks for the centralities.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 */
public class GraphPreconditionChecks {

	private static String errorMessage = "";
    //private static final String OPERATION_NAME = "Checking Preconditions";

	public static String getErrorMessage(){
		return errorMessage;
	}
	
    public static boolean checkPreconditionsLFSimple(CentiLib<?,?> centibin) {
        return checkPreconditions(centibin, false, false, false, false, true, true);
    }

    public static boolean checkPreconditionsSimple(CentiLib<?,?> centibin) {
        return checkPreconditions(centibin, false, false, false, false, false, true);
    }

    public static boolean checkPreconditionsConUndirLFSimple(CentiLib<?,?> centibin) {
        return checkPreconditions(centibin, true, false, true, false, true, true);
    }
    
    public static boolean checkPreconditionsUndirSimple(CentiLib<?,?> centibin) {
        return checkPreconditions(centibin, false, false, true, false, false, true);
    }
    
    public static boolean checkPreconditionsConUndirSimple(CentiLib<?,?> centibin) {
        return checkPreconditions(centibin, true, false, true, false, false, true);
    }

    public static boolean checkPreconditionsConDirLFSimple(CentiLib<?,?> centibin) {
        return checkPreconditions(centibin, true, false, false, true, true, true);
    }

    public static boolean checkPreconditionsConDirSimple(CentiLib<?,?> centibin) {
        return checkPreconditions(centibin, true, false, false, true, false, true);
    }
    
    public static boolean checkPreconditionsConLFSimple(CentiLib<?,?> centibin) {
        return checkPreconditions(centibin, true, false, false, false, true, true);
    }

    public static boolean checkPreconditionsConSimple(CentiLib<?,?> centibin) {
        return checkPreconditions(centibin, true, false, false, false, false, true);
    }

    public static boolean checkPreconditionsStrongConLFSimple(CentiLib<?,?> centibin) {
        return checkPreconditions(centibin, true, true, false, false, true, true);
    }
    
    public static boolean checkPreconditionsStrongConSimple(CentiLib<?,?> centibin) {
        return checkPreconditions(centibin, true, true, false, false, false, true);
    }
    
    public static boolean checkPreconditionsStrongConDirLFSimple(CentiLib<?,?> centibin) {
        return checkPreconditions(centibin, true, true, false, true, true, true);
    }

    public static boolean checkPreconditionsStrongConDirSimple(CentiLib<?,?> centibin) {
    	return checkPreconditions(centibin, true, true, false, true, false, true);
    }

    
    /**
     * checks the conditions for the Graph
     * 
     * @param checkConnected:
     *            if true, graph is checked to be connected, else it does not
     *            matter
     * @param checkStrongConnected:
     *            if true, graph is checked to be connected, else it does not
     *            matter
     * @param checkUndirected:
     *            if true, graph is checked to be undirected, else it has to be
     *            directed
     * @param checkDirected:
     *            if true, graph is checked to be undirected, else it has to be
     *            directed
     * @param checkLoopFree:
     *            if true, graph is checked to be loopfree, else it does not
     *            matter
     * @param checkGraphSimple:
     *            if true, graph is checked to be simple, else it does not
     *            matter
     * @return boolean preconditions are fulfilled
     */
    public static boolean checkPreconditions(CentiLib<?,?> centibin, boolean checkConnected, boolean checkStrongConnected,
            boolean checkUndirected, boolean checkDirected, boolean checkLoopFree, boolean checkGraphSimple) {
    	return checkPreconditions(centibin.getCurrentGraph(), checkConnected, checkStrongConnected, checkUndirected, checkDirected, checkLoopFree, checkGraphSimple);
    }

    public static boolean checkPreconditions(Graph<?,?> graph, boolean checkConnected, boolean checkStrongConnected,
            boolean checkUndirected, boolean checkDirected, boolean checkLoopFree, boolean checkGraphSimple) {

    	
    	boolean result = true;
    	errorMessage = "      ";
    	    	
        if(graph == null || graph.getVertexCount()==0){
        	errorMessage += "No Vertices found.";
        	return false;
        }


        if (checkUndirected && !GraphCachingUtils.isUndirected(graph)) {
        	if(!errorMessage.equals("      ")) errorMessage +="\n      ";
            errorMessage += "Graph is directed.";
            result = false;

        }

        if (checkDirected && !GraphCachingUtils.isDirected(graph)) {
        	if(!errorMessage.equals("      ")) errorMessage +="\n      ";
            errorMessage += "Graph is undirected.";
            result = false;

        }

        if (checkLoopFree && GraphCachingUtils.containsSelfLoops(graph)) {
        	if(!errorMessage.equals("      ")) errorMessage +="\n      ";
            errorMessage += "Graph contains loops.";
            result = false;
        }

        if (checkGraphSimple && GraphCachingUtils.containsParallelEdges(graph)) {
        	if(!errorMessage.equals("      ")) errorMessage +="\n      ";
            errorMessage += "Graph contains parallel edges.";
            result = false;
        }

        if (checkConnected && !GraphCachingUtils.isConnected(graph)) {
            if(!errorMessage.equals("      ")) errorMessage +="\n      ";
            errorMessage += "Graph is not connected.";
            result = false;
        }

        if (checkStrongConnected && !GraphCachingUtils.isStrongConnected(graph)) {
        	if(!errorMessage.equals("      ")) errorMessage +="\n      ";
            errorMessage += "Graph is not strong connected.";
            result = false;

        }

        if(errorMessage.equals("      ")) errorMessage = "";
        return result;
        
     }
}
