package de.ipk_gatersleben.ag_nw.centilib.utils;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

import edu.uci.ics.jung.algorithms.shortestpath.Distance;
import edu.uci.ics.jung.algorithms.shortestpath.UnweightedShortestPath;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.util.EdgeType;
import edu.uci.ics.jung.graph.util.Pair;

/**
 * Tools for checking properties of a graph
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 */
public class GraphProperties {

	
	/**
	 * Checks to see whether the graph is strong connected
	 * 
	 * @param g
	 *            the graph
	 * @return Return true if yes, false if no
	 */
	
	public static <V,E> boolean isStrongConnected(Graph<V,E> g) {

		UnweightedShortestPath<V,E> d = new UnweightedShortestPath<V,E>(g);
		return isStrongConnected(g,d);
	}
	
	public static <V,E> boolean isStrongConnected(Graph<V,E> graph, Distance<V> distance){

		Collection<V> visit = graph.getVertices();
		for(V v : graph.getVertices()){
			for(V w : graph.getVertices()){
				if(v == w) continue;
				if(distance.getDistance(v, w) == null){
					return false;
				}
			}
		}
		return true;
	}
	
	public static <V,E> boolean isConnected(Graph<V,E> graph){
		
		if(graph.getVertices().isEmpty())
			return true;
		
		Collection<V> 
			visited = new HashSet<V>(),
			seen = new HashSet<V>(),
			tempseen = new HashSet<V>(),
			neighbors;
		
		//'random' start node
		seen.add(graph.getVertices().iterator().next());
		
		//Breadth-first search
		while(!seen.isEmpty()){
			for(V v : seen){
				visited.add(v);
				
				for(V w : graph.getNeighbors(v))
					//only add nodes, that aren't known now
					if(!visited.contains(w) && !seen.contains(w))
						tempseen.add(w);
			}

			seen.clear();
			//swap seen-tempseen
			neighbors = seen;
			seen = tempseen;
			tempseen = neighbors;
		}
		
		if(visited.size() == graph.getVertexCount())
			return true;
		else return false;
	}
	
	public static <V,E> boolean isDirected(Graph<V,E> graph) {
		
		if(graph.getEdges(EdgeType.UNDIRECTED).size() != 0)
			return false;
		return true;
	}

	public static boolean isUndirected(Graph<?,?> graph) {
		
		if(graph.getEdges(EdgeType.DIRECTED).size() != 0)
			return false;
		return true;
	}

	/*
	 * Only for Edges with two endpoints 
	 */
    public static <V,E> boolean containsSelfLoops(Graph<V,E> graph) {
    	
    	for(E e : graph.getEdges()){
    		Iterator<V>	it = graph.getIncidentVertices(e).iterator();
    		if(it.next().equals(it.next())){
    			return true;
    		}
    	}		
    	return false;
    }
    
    public static <V,E> boolean containsParallelEdges(Graph<V,E> graph) {
		
    	for (E e : graph.getEdges()) {
			Pair<V> endpoints = graph.getEndpoints(e);
			V anEndPoint = endpoints.getFirst();
			V anotherEndPoint = endpoints.getSecond();
			Collection<E> edges = graph.findEdgeSet(anEndPoint,anotherEndPoint);
			for(E e1 : edges){
				for(E e2 : edges){
					if(e1 == e2) continue;
					Pair<V> p1 = graph.getEndpoints(e1),
						p2 = graph.getEndpoints(e2);
					if(p1.getFirst() == p2.getFirst() && p1.getSecond() == p2.getSecond())
						return true;
				}
			}
		}

		return false;
    }
}
