package de.ipk_gatersleben.ag_nw.centilib.utils;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.Stack;

import edu.uci.ics.jung.algorithms.cluster.WeakComponentClusterer;
import edu.uci.ics.jung.graph.Graph;

public class GraphUtils {
	
	public static <V, E> Set<Set<V> > getConnectedComponents(
			Graph<V, E> graph) {
		
		WeakComponentClusterer<V, E> components = new WeakComponentClusterer<V, E>();
		
		if(graph != null) return components.transform(graph);
		return null;
	}
	
	public static <V,E> Set<E> getAllEdgesOfVertices(Graph<V,E> graph, Set<V> vertices){
		//returns all edges connected to the given vertices
		
		if(graph == null && vertices== null) return null;
		Set<E> edges = new HashSet<E>();
		for(V vertex : vertices)
			if(graph.getIncidentEdges(vertex) != null)
				edges.addAll(graph.getIncidentEdges(vertex));
		
		return edges;
			
	}
	
	public static <V,E> Set<E> getEdgesBetweenVertices(Graph<V,E> graph, Set<V> vertices){
		//returns all edges which endpoints are contained in the given Set
		if(graph == null && vertices== null) return null;
		
		Set<E> edges = new HashSet<E>();
				
		for(V vertex : vertices)
			//for all vertices in the given Set
			for(E edge : graph.getIncidentEdges(vertex))
				//look for all neighbors contained in the Set and add the appropriate edge to the result
				if(vertices.contains(graph.getOpposite(vertex,edge)))
					edges.add(edge);
		return edges;
	}
	
	public static <V,E> Set<Set<V> > getStrongConnectedComponents(Graph<V,E> graph){
		//This is an implementation of Tarjans Algorithm for strongly connected components
		
		if(graph == null) return null;
		
		Map<V, Integer> indices = new HashMap<V, Integer>();
		Map<V, Integer> lowlinks = new HashMap<V, Integer>();
		Stack<V> stack = new Stack<V>();
		Integer index = new Integer(0);
		Set<Set<V > > SCC = new HashSet<Set<V> >();
		
		for(V vertex : graph.getVertices()){
			if(indices.get(vertex) == null)
				tarjan(graph,vertex,indices,lowlinks,index,stack, SCC);
		}
		
		return SCC;
	}
	
	private static <V,E> int tarjan(
			Graph<V,E> graph,
			V vertex, 
			Map<V, Integer> indices, 
			Map<V, Integer> lowlinks, 
			Integer index, 
			Stack<V> stack, 
			Set<Set<V> > SCC){
		
		indices.put(vertex, index);
		lowlinks.put(vertex, index);
		index++;
		stack.push(vertex);
		
		for(V suc : graph.getSuccessors(vertex)){
			if(indices.get(suc) == null){
				index = tarjan(graph,suc,indices,lowlinks,index,stack,SCC);
				//vertex.lowlink = min(vertex.lowlink, suc.lowlink)
				lowlinks.put(vertex, lowlinks.get(vertex)<lowlinks.get(suc) ? lowlinks.get(vertex) : lowlinks.get(suc));
			}
			
			else
				if(stack.contains(suc))
					//vertex.lowlink = min(vertex.lowlink, suc.index)
					lowlinks.put(vertex, lowlinks.get(vertex)<indices.get(suc)? lowlinks.get(vertex) : indices.get(suc));
		}
		
		V temp;
		if(indices.get(vertex).equals(lowlinks.get(vertex))){
			Set<V> comp = new HashSet<V>();
			do{
				temp = stack.pop();
				comp.add(temp);
			}while(temp != vertex);
			
			SCC.add(comp);
		}
		return index;
	}
}
