package de.ipk_gatersleben.ag_nw.centilib.utils;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.collections15.Transformer;

import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.util.EdgeType;

/**
 * This class normalizes the outgoing edge weights of each vertex,
 * such that they sum to 1.
 * Throws an IllegalArgumentException() if the graph is not directed.
 * 
 * @author Johannes Graessler
 *
 * @param <V> Vertex Type
 * @param <E> Edge Type
 */
public class UniformWeightedOutEdges<V,E> implements Transformer<E, Double> {

	Map<E,Double> output = new HashMap<E,Double>();
	
	public UniformWeightedOutEdges(Graph<V,E> graph, Map<E,? extends Number> edgeWeights) {
		normalizeEdgeWeights(graph, edgeWeights);
	}

	@Override
	public Double transform(E edge) {
		// TODO Auto-generated method stub
		return output.get(edge);
	}

	private void normalizeEdgeWeights(Graph<V,E> graph, Map<E,? extends Number> weights) throws IllegalArgumentException{
		
		if(graph == null || weights == null)
			throw new IllegalArgumentException("No graph or no edge weights specified");
		if(graph.getEdges(EdgeType.UNDIRECTED).size() != 0)
			throw new IllegalArgumentException("Graph is not directed.");
		
		
		for(V v : graph.getVertices()){
			Collection<E> edges = graph.getOutEdges(v);
			if(edges == null || edges.size() == 0)
				continue;
			
			Double sum = 0.0;
			
			for(E edge : edges)
				sum += weights.get(edge).doubleValue();
			
			if(sum == 0)
				sum = 1.0;
			for(E edge : edges)
				output.put(edge, weights.get(edge).doubleValue()/sum);
		}
	}
}
